VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "DBaseObj"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'   CONSTANTS
'----------------------------------------------------------------------------------

Private Const OBJ_NAME = "DBaseObj"

'----------------------------------------------------------------------------------
'   OBJECTS
'----------------------------------------------------------------------------------

Private poDBase As Database             ' The database associated with this class
Private poData As Data                  ' A link to a data control
Private poError As New ErrorObj         ' The error handling routines

Private poVList As Object               ' Reference to a list control
Private poVForm As Form                 ' Reference to the form the list is on

'----------------------------------------------------------------------------------
'   STRUCTURES
'----------------------------------------------------------------------------------

' None

'----------------------------------------------------------------------------------
'   VARIABLES
'----------------------------------------------------------------------------------

Private ptaTable() As extRecordSetType  ' The recordsets open in this class
Private piNumTables As Integer          ' Number of recordsets open

Private ptTempSet As Recordset          ' Temporary recordset for assorted querying

Private piCurTable As Integer           ' The current table index selected

Private pbDBaseOpen As Boolean          ' Determines whether the database was
                                        ' openned
Private pbDataUsed As Boolean           ' Shows whether a data control or
                                        ' database object was used in initialization
Private pbInitialized As Boolean        ' Used to keep the class from being initialized twice

'
'   xAddRule
'   PURPOSE:
'       - Adds a validation rule to the current column in the current recordset
'   PARAMETERS:
'       - Rule (integer)
'           - The rule to associate with the current column
'       - Value (optional long)
'           - The value to associate with the rule
'           - If not given, defaults to 0
'   RETURNS:
'       - None
Public Sub xAddRule(riRule%, Optional rvntValue)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xAddRule"
    poError.xReset

    Dim iCurColumn%
    Dim lValue&, m%, n%
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to add a rule if no recordsets exist."
        Exit Sub
    End If

    If (ptaTable(piCurTable).iNumColumns <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to add a rule if no columns exist."
        Exit Sub
    End If
    
    If IsMissing(rvntValue) Then
        lValue = 0
    Else
        lValue = CLng(rvntValue)
    End If
    
    iCurColumn = ptaTable(piCurTable).iCurColumn

'   Search for the given rule in the current rules
    m = ptaTable(piCurTable).taColumn(iCurColumn).iNumRules
    For n = 1 To m
        If (ptaTable(piCurTable).taColumn(iCurColumn).taRule(n).iRule = riRule) Then
'           Rule was found, change the value
            ptaTable(piCurTable).taColumn(iCurColumn).taRule(n).lValue = lValue
            Exit Sub
        End If
    Next n
    If (n > m) Then
'       Rule was not found, add it
        ptaTable(piCurTable).taColumn(iCurColumn).iNumRules = n
        
        ReDim Preserve ptaTable(piCurTable).taColumn(iCurColumn).taRule(1 To n) As RuleType
        
        ptaTable(piCurTable).taColumn(iCurColumn).taRule(n).iRule = riRule
        ptaTable(piCurTable).taColumn(iCurColumn).taRule(n).lValue = lValue
    End If
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub

'   xAddIndex
'   PURPOSE:
'       - Adds an index to the current recordset
'   PARAMETERS:
'       - IndexName (string)
'           - Name of the index (user-defined)
'       - Unique (boolean)
'           - Determines whether the index is unique or not
'   RETURNS:
'       - None
Public Sub xAddIndex(rsIndexName$, rbUnique As Boolean)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xAddIndex"
    poError.xReset

    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to add an index if no recordsets exist."
        Exit Sub
    End If
    
    Dim m%, n%
    
'   Search for the given index in the current indeces
    m = ptaTable(piCurTable).iNumIndeces
    For n = 1 To m
        If (ptaTable(piCurTable).taIndex(n).sName = rsIndexName) Then
'           Index was found, change its value
            ptaTable(piCurTable).iCurIndex = n
            
            ptaTable(piCurTable).taIndex(n).bUnique = rbUnique
            ptaTable(piCurTable).taIndex(n).iNumFields = 0
            Exit Sub
        End If
    Next n
    If (n > m) Then
'       Index was not found, add it
        ptaTable(piCurTable).iNumIndeces = n
        
        ReDim Preserve ptaTable(piCurTable).taIndex(1 To n) As IndexType
        
        ptaTable(piCurTable).taIndex(n).sName = rsIndexName
        ptaTable(piCurTable).taIndex(n).bUnique = rbUnique
        
        ptaTable(piCurTable).iCurIndex = n
    End If
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   gsIndexField (Let)
'   PURPOSE:
'       - Adds a field to the current index in the current recordset
'       - NOTE: There is a difference between the first index field added and
'               any other index fields in that index:
'               The first index field is the one changed if a duplicate key is found
'               during an AddNew operation
'   PARAMETERS:
'       - FieldName (string)
'           - Name of the field
'   RETURNS:
'       - None
Public Property Let gsIndexField(rsFieldName$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsIndexField (Let)"
    poError.xReset

    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to add an index field if no recordsets exist."
        Exit Property
    End If
    
    If (ptaTable(piCurTable).iNumIndeces <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to add an index field if no indeces exist."
        Exit Property
    End If

    If Not xFieldExists(rsFieldName) Then
        poError.xSet sCurrentRoutine, "Field '" & rsFieldName & "' doesn't exist in the current recordset."
        Exit Property
    End If
    
    Dim iCurIndex%, m%, n%

'   Search for the given field in the current fields
    iCurIndex = ptaTable(piCurTable).iCurIndex
    
    m = ptaTable(piCurTable).taIndex(iCurIndex).iNumFields
    For n = 1 To m
        If (ptaTable(piCurTable).taIndex(iCurIndex).saField(n) = rsFieldName) Then
'           Field was found
            Exit Property
        End If
    Next n
    If (n > m) Then
'       Field was not found, add it
        ptaTable(piCurTable).taIndex(iCurIndex).iNumFields = n
        
        ReDim Preserve ptaTable(piCurTable).taIndex(iCurIndex).saField$(1 To n)
        
        ptaTable(piCurTable).taIndex(iCurIndex).saField(n) = rsFieldName
    End If
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property



'   gsColumnList (Let)
'   PURPOSE:
'       - Sets the list of possible values for the current column
'   PARAMETERS:
'       - List (string)
'           - The possible values for this list
'   RETURNS:
'       - None
Public Property Let gsColumnList(rsList$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsColumnList (Let)"
    poError.xReset

    Dim iCurColumn%
    Dim iNumFields%, m%, n%
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to set the list if no recordsets exist."
        Exit Property
    End If

    If (ptaTable(piCurTable).iNumColumns <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to set the list if no columns exist."
        Exit Property
    End If
        
    iCurColumn = ptaTable(piCurTable).iCurColumn
    ptaTable(piCurTable).taColumn(iCurColumn).sList = rsList
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property

'   gsDefault (Let)
'   PURPOSE:
'       - Sets the default value for this field
'   PARAMETERS:
'       - List (string)
'           - The possible values for this list
'   RETURNS:
'       - None
Public Property Let gsDefault(rsDefault$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsDefault (Let)"
    poError.xReset

    Dim iCurColumn%
    Dim iNumFields%, m%, n%
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to set the default if no recordsets exist."
        Exit Property
    End If

    If (ptaTable(piCurTable).iNumColumns <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to set the default if no columns exist."
        Exit Property
    End If
        
    iCurColumn = ptaTable(piCurTable).iCurColumn
    ptaTable(piCurTable).taColumn(iCurColumn).sDefault = rsDefault
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property


'   gsValidList (Let)
'   PURPOSE:
'       - Sets the list of valid values for the current column
'   PARAMETERS:
'       - List (string)
'           - The possible values for this list
'   RETURNS:
'       - None
Public Property Let gsValidList(rsList$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsValidList (Let)"
    poError.xReset

    Dim iCurColumn%
    Dim iNumFields%, m%, n%
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to set the list if no recordsets exist."
        Exit Property
    End If

    If (ptaTable(piCurTable).iNumColumns <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to set the list if no columns exist."
        Exit Property
    End If
        
    iCurColumn = ptaTable(piCurTable).iCurColumn
    ptaTable(piCurTable).taColumn(iCurColumn).sValidList = rsList
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property



'   gsColumnList (Get)
'   PURPOSE:
'       - Returns the list of possible values for the current column
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - The possible values for this list
Public Property Get gsColumnList$()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsColumnList (get)"
    poError.xReset

    Dim iCurColumn%
    Dim iNumFields%, m%, n%
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to set the list if no recordsets exist."
        Exit Property
    End If

    If (ptaTable(piCurTable).iNumColumns <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to set the list if no columns exist."
        Exit Property
    End If
        
    iCurColumn = ptaTable(piCurTable).iCurColumn
    gsColumnList = ptaTable(piCurTable).taColumn(iCurColumn).sList
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property




'   gsValidList (Get)
'   PURPOSE:
'       - Returns the list of valid values for the current column
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - The valid values for this list
Public Property Get gsValidList$()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsValidList (Get)"
    poError.xReset

    Dim iCurColumn%
    Dim iNumFields%, m%, n%
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to set the list if no recordsets exist."
        Exit Property
    End If

    If (ptaTable(piCurTable).iNumColumns <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to set the list if no columns exist."
        Exit Property
    End If
        
    iCurColumn = ptaTable(piCurTable).iCurColumn
    gsValidList = ptaTable(piCurTable).taColumn(iCurColumn).sValidList
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property




'   gsColumnName(Get)
'   PURPOSE:
'       - Returns the name of the current column
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - The name of the column
Public Property Get gsColumnName$()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsColumnName (Get)"
    poError.xReset

    Dim iCurColumn%
    Dim iNumFields%, m%, n%
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to get the column name if no recordsets exist."
        Exit Property
    End If

    If (ptaTable(piCurTable).iNumColumns <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to get the column name if no columns exist."
        Exit Property
    End If
        
    iCurColumn = ptaTable(piCurTable).iCurColumn
    gsColumnName = ptaTable(piCurTable).taColumn(iCurColumn).sName
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property




'   xAddColumn
'   PURPOSE:
'       - Adds a column to the current recordset
'   PARAMETERS:
'       - ColumnField (string)
'           - The field name of the column
'       - ColumnName (optional string)
'           - The header on the column
'           - This is used to distinguish between two columns which use
'             the same column field
'   RETURNS:
'       - None
Public Sub xAddColumn(rsColumnField$, Optional rvntColumnName)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xAddColumn"
    poError.xReset

    Dim lValue&, m%, n%
    Dim sColumnName$
    
    If IsMissing(rvntColumnName) Then
        sColumnName = LCase$(rsColumnField)
    Else
        sColumnName = LCase$(rsColumnField & "(" & CStr(rvntColumnName) & ")")
    End If
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to add a column if no recordsets exist."
        Exit Sub
    End If

'   Search for the given column in the column listing
    m = ptaTable(piCurTable).iNumColumns
    For n = 1 To m
        If (LCase$(ptaTable(piCurTable).taColumn(n).sName) = sColumnName) Then
'           Column was found, erase the old column information
            ptaTable(piCurTable).iCurColumn = n
            ptaTable(piCurTable).taColumn(n).iNumRules = 0
            ptaTable(piCurTable).taColumn(n).sList = ""
            ptaTable(piCurTable).taColumn(n).sDefault = ""
            Exit Sub
        End If
    Next n
    If (n > m) Then
'       Column was not found, add it
        ptaTable(piCurTable).iNumColumns = n
        
        ReDim Preserve ptaTable(piCurTable).taColumn(1 To n) As ColumnType
        
        ptaTable(piCurTable).taColumn(n).sName = sColumnName
        ptaTable(piCurTable).taColumn(n).sField = rsColumnField
        ptaTable(piCurTable).iCurColumn = n
    End If
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   xInitialize
'   PURPOSE:
'       - Starts up the database, and opens all the tables for I/O
'       - Sets up the database to use a data control or database object
'   PARAMETERS:
'       - DBaseName (string)
'           - The name of the database to open
'       - Data (optional data control)
'           - A link to a data control
'           - If left out the database object is used
'   RETURNS:
'       - None
Public Sub xInitialize(rsDBaseName$, Optional rvntData)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xInitialize"
    poError.xReset
        
    If pbInitialized Then
        poError.xSet sCurrentRoutine, "You can not initialize object twice."
        Exit Sub
    End If
    pbInitialized = True
                
    If IsMissing(rvntData) Then
        Set poDBase = OpenDatabase(rsDBaseName)
        pbDBaseOpen = True
        pbDataUsed = False
    Else
        Set poDBase = OpenDatabase(rsDBaseName)
        Set poData = rvntData
        
        poData.DatabaseName = rsDBaseName
        pbDBaseOpen = True
        pbDataUsed = True
    End If
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub



'   xAddTable
'   PURPOSE:
'       - Opens a table in the database
'       - Use SQL to define the recordset or a Table name
'   PARAMETERS:
'       - TableName (string)
'           - The name of the table to open
'           - if SQL is given this is a user defined name given to the recordset
'       - Required (string)
'           - The required WHERE statement to make this recordset work
'           - "" is considered NO REQUIRED WHERE
'       - SQL (optional string)
'           - A SQL statement defining the recordset
'           - If missing, the table name is used instead
'   RETURNS:
'       - None
Public Sub xAddTable(rsTableName$, rsRequired$, Optional rvntSQL)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xAddTable"
    poError.xReset
    
    Dim bErrorChecked As Boolean: bErrorChecked = False
    
    Dim n%
    
    If pbDataUsed Then
'       Data controls are allowed only one recordset open at a time
        ReDim ptaTable(1 To 1) As extRecordSetType
                
        ptaTable(1).sRequiredFind = rsRequired
        If IsMissing(rvntSQL) Then
            ptaTable(1).sSQL = "SELECT * FROM " & rsTableName
        Else
            ptaTable(1).sSQL = CStr(rvntSQL)
        End If
                
        ptaTable(1).sName = rsTableName
        piNumTables = 1
        gsCurTable = rsTableName
        
        xQuery
        xRefresh
        xMoveFirst
    Else
        If Not pbDBaseOpen Then
            poError.xSet sCurrentRoutine, "Unable to open table with database uninitialized."
            Exit Sub
        End If
        
'       Check to see if that table is already open
        If (piNumTables > 0) Then
            For n = 1 To UBound(ptaTable)
                If (LCase$(ptaTable(n).sName) = LCase$(rsTableName)) Then
                    piCurTable = n
                    Exit For
                End If
            Next n
            If (n > UBound(ptaTable)) Then
                GoTo CREATE_NEW_RECORDSET
            End If
        Else
CREATE_NEW_RECORDSET:
            piNumTables = piNumTables + 1
            ReDim Preserve ptaTable(1 To piNumTables) As extRecordSetType
            
            ptaTable(piNumTables).sRequiredFind = rsRequired
            If IsMissing(rvntSQL) Then
                ptaTable(piNumTables).sSQL = "SELECT * FROM " & rsTableName
            Else
                ptaTable(piNumTables).sSQL = CStr(rvntSQL)
            End If
            ptaTable(piNumTables).sName = rsTableName
            gsCurTable = rsTableName
            
            xQuery
            xRefresh
            xMoveFirst
        End If
    End If
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   gsCurTable (Let)
'   PURPOSE:
'       - Sets the current Table of the database
'   PARAMETERS:
'       - rsTableName (string)
'           - The name of the table
'   RETURNS:
'       - None
Public Property Let gsCurTable(rsTableName$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsCurTable (Let)"
    poError.xReset
    
    Dim n%
    
    If (piNumTables > 0) Then
        For n = 1 To UBound(ptaTable)
            If (LCase$(ptaTable(n).sName) = LCase$(rsTableName)) Then
                piCurTable = n
                
'               Refresh the recordset from the database
                xQuery
                
                Exit For
            End If
        Next n
        If (n > UBound(ptaTable)) Then
            poError.xSet sCurrentRoutine, rsTableName & " must be opened before it can be selected."
            Exit Property
        End If
    Else
        poError.xSet sCurrentRoutine, rsTableName & " must be opened before it can be selected."
        Exit Property
    End If
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property




'   gsCurColumn (Let)
'   PURPOSE:
'       - Sets the current column of the current recordset
'   PARAMETERS:
'       - rsColumnName (string)
'           - The name of the column
'   RETURNS:
'       - None
Public Property Let gsCurColumn(rsColumnName$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsCurColumn (Let)"
    poError.xReset
    
    Dim n%
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to set the column if no recordsets exist."
        Exit Property
    End If
    
    If (ptaTable(piCurTable).iNumColumns <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to set the column if no columns exist."
        Exit Property
    End If
        
    For n = 1 To UBound(ptaTable(piCurTable).taColumn)
        If (LCase$(ptaTable(piCurTable).taColumn(n).sName) = LCase$(rsColumnName)) Then
            ptaTable(piCurTable).iCurColumn = n
            Exit For
        End If
    Next n
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property





'   xIsString
'   PURPOSE:
'       - Determines whether the given field is a string value or not
'   PARAMETERS:
'       - FieldName (string)
'           - The field to check
'   RETURNS:
'       - (boolean)
'           - TRUE : The field is a string value
'           - FALSE: The field is not a string value
Public Function xIsString(rsFieldName$) As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xIsString"
    poError.xReset

    If pbDataUsed Then
        Select Case poData.Recordset(rsFieldName).Type
        Case dbText, dbMemo
            xIsString = True
        Case Else
            xIsString = False
        End Select
    Else
        Select Case ptaTable(piCurTable).tRecordset(rsFieldName).Type
        Case dbText, dbMemo
            xIsString = True
        Case Else
            xIsString = False
        End Select
    End If
    
    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

'   xIsDate
'   PURPOSE:
'       - Determines whether the given field is a date value or not
'   PARAMETERS:
'       - FieldName (string)
'           - The field to check
'   RETURNS:
'       - (boolean)
'           - TRUE : The field is a date value
'           - FALSE: The field is not a date value
Public Function xIsDate(rsFieldName$) As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xIsDate"
    poError.xReset

    If pbDataUsed Then
        xIsDate = (poData.Recordset(rsFieldName).Type = dbDate)
    Else
        xIsDate = (ptaTable(piCurTable).tRecordset(rsFieldName).Type = dbDate)
    End If
    
    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function


'   xQuery
'   PURPOSE:
'       - Requeries the database to make a new recordset
'       - Uses SortCriteria, FindCriteria, RequiredFind
'   PARAMETERS:
'       - UseTempSet (optional boolean)
'           - If given, the TempSet is the receiver of the query
'       - UseFunction (optional string)
'           - Only applicable if used with UseTempSet
'           - All functions are available for use
'       - UseField (optional string)
'           - Only applicable if used with UseFunction
'           - Provides a field to use with the given function
'           - Defaults to '*' if not given
'   RETURNS:
'       - None
Public Sub xQuery(Optional rvntUseTempSet, Optional rvntUseFunction, Optional rvntUseField)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xQuery"
    poError.xReset
    
    Dim sBuffer$

    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to find a record if no recordsets are open."
        Exit Sub
    End If

'   Create a query to handle the find
    If IsMissing(rvntUseFunction) Then
        sBuffer = ptaTable(piCurTable).sSQL
    ElseIf IsMissing(rvntUseField) Then
        sBuffer = "SELECT " & Format$(rvntUseFunction) & "(*) as N " & Mid$(ptaTable(piCurTable).sSQL, InStr(UCase$(ptaTable(piCurTable).sSQL), "FROM"))
    Else
        sBuffer = "SELECT " & Format$(rvntUseFunction) & "(" & Format$(rvntUseField) & ") as N " & Mid$(ptaTable(piCurTable).sSQL, InStr(UCase$(ptaTable(piCurTable).sSQL), "FROM"))
    End If
        
    If ((ptaTable(piCurTable).sFindCriteria <> "") And (ptaTable(piCurTable).sRequiredFind <> "")) Then
        sBuffer = sBuffer & " WHERE " & ptaTable(piCurTable).sFindCriteria & " AND " & ptaTable(piCurTable).sRequiredFind
    ElseIf (ptaTable(piCurTable).sFindCriteria <> "") Then
        sBuffer = sBuffer & " WHERE " & ptaTable(piCurTable).sFindCriteria
    ElseIf (ptaTable(piCurTable).sRequiredFind <> "") Then
        sBuffer = sBuffer & " WHERE " & ptaTable(piCurTable).sRequiredFind
    End If
    
    If IsMissing(rvntUseFunction) Then
        If (ptaTable(piCurTable).sSortCriteria <> "") Then
            sBuffer = sBuffer & " ORDER BY " & ptaTable(piCurTable).sSortCriteria
        End If
    End If
    
    If Not IsMissing(rvntUseTempSet) Then
        ptTempSet.Close
        Set ptTempSet = poDBase.OpenRecordset(sBuffer)
    ElseIf pbDataUsed Then
        ptaTable(piCurTable).tRecordset.Close
        Set ptaTable(piCurTable).tRecordset = poDBase.OpenRecordset(sBuffer)
        
        poData.RecordSource = sBuffer
    Else
        ptaTable(piCurTable).tRecordset.Close
        Set ptaTable(piCurTable).tRecordset = poDBase.OpenRecordset(sBuffer)
    End If

    Exit Sub
ERRORTRAP:
    Select Case Err.Number
    Case 91, 3420     ' Recordset not set, Object not set
'       Caused by the first call to this routine when the recordset is closed
'       even though it was never openned before hand
    Case Else
        Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    Resume Next
End Sub

Private Sub Class_Initialize()
    pbInitialized = False
End Sub

Private Sub Class_Terminate()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".Class_Terminate"
    poError.xReset
     
    Dim n%
     
    If pbDBaseOpen Then
        If pbDataUsed Then poData.Recordset.Close

'       Close the open tables
        If (piNumTables > 0) Then
            For n = 1 To UBound(ptaTable)
                ptaTable(n).tRecordset.Close
            Next n
        End If
        ptTempSet.Close
        
'       Close the database
        poDBase.Close
    End If
    
    Set poDBase = Nothing
    Set poData = Nothing
    Set poError = Nothing
    
    Exit Sub
ERRORTRAP:
    Select Case Err.Number
    Case 91 ' Object not set
'       Ignore this error, it occurs if this class never uses ptTempSet
    Case Else
        Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    Resume Next
End Sub



'   xGet
'   PURPOSE:
'       - Returns the given field from the current record in the current table
'   PARAMETERS:
'       - rsField (string)
'           - The field to return
'   RETURNS:
'       - (variant)
'           - The value of the field
Public Function xGet(rsField$) As Variant
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xGet"
    poError.xReset
        
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to get a field if no recordsets are open."
        Exit Function
    End If
    
    If pbDataUsed Then
        xGet = poData.Recordset(rsField).Value
    Else
        xGet = ptaTable(piCurTable).tRecordset(rsField).Value
    End If
    
    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

'   xSet
'   PURPOSE:
'       - Sets the given Field to the given value in the current table
'   PARAMETERS:
'       - rsField (string)
'           - The field to return
'       - (variant)
'           - The value of the field
'   RETURNS:
'       - None
Public Sub xSet(rsField$, rvntValue)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xSet"
    poError.xReset
        
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to set a field if no recordsets are open."
        Exit Sub
    End If
    
    If pbDataUsed Then
        poData.Recordset.Edit
            poData.Recordset(rsField).Value = rvntValue
        poData.Recordset.Update
    Else
        ptaTable(piCurTable).tRecordset.Edit
            ptaTable(piCurTable).tRecordset(rsField).Value = rvntValue
        ptaTable(piCurTable).tRecordset.Update
    End If
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   xFind
'   PURPOSE:
'       - Sets the recordset up to show only those records which match the
'         criteria
'   PARAMETERS:
'       - Criteria (string)
'           - The criteria to find the record
'   RETURNS:
'       - None
Public Sub xMassFind(rsCriteria$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xFind"
    poError.xReset
            
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to find a record if no recordsets are open."
        Exit Sub
    End If
        
    ptaTable(piCurTable).sFindCriteria = rsCriteria
    xQuery
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub

'   xFind
'   PURPOSE:
'       - Attempts to locate a record which matchs the given find criteria
'   PARAMETERS:
'       - Criteria (string)
'           - The criteria to find the record
'   RETURNS:
'       - (boolean)
'           - TRUE : A record was located
'           - FALSE: A record was not located
Public Function xFind(rsCriteria$) As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xFind"
    poError.xReset
            
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to find a record if no recordsets are open."
        Exit Function
    End If
            
    ptaTable(piCurTable).sSngFindCriteria = rsCriteria
    ptaTable(piCurTable).tRecordset.FindFirst rsCriteria
    xFind = Not ptaTable(piCurTable).tRecordset.NoMatch
    
    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function


'   xFindNext
'   PURPOSE:
'       - Attempts to locate the next record in a find listing
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (boolean)
'           - TRUE : A record was located
'           - FALSE: A record was not located
Public Function xFindNext() As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xFindNext"
    poError.xReset
        
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to find a record if no recordsets are open."
        Exit Function
    End If
        
    ptaTable(piCurTable).tRecordset.FindNext ptaTable(piCurTable).sSngFindCriteria
    xFindNext = Not ptaTable(piCurTable).tRecordset.NoMatch
    
    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function


'   xFindPrev
'   PURPOSE:
'       - Attempts to locate the previous record in a find listing
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (boolean)
'           - TRUE : A record was located
'           - FALSE: A record was not located
Public Function xFindPrev() As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xFindPrev"
    poError.xReset
        
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to find a record if no recordsets are open."
        Exit Function
    End If
        
    ptaTable(piCurTable).tRecordset.FindPrevious ptaTable(piCurTable).sSngFindCriteria
    xFindPrev = Not ptaTable(piCurTable).tRecordset.NoMatch
    
    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function




'   xRefresh
'   PURPOSE:
'       - Refreshs the database object
'   PARAMETERS:
'       - None
'   RETURNS:
'       - None
Public Sub xRefresh()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xRefresh"
    poError.xReset
    
    Dim n%, iIndex%
    If pbDataUsed Then
        poData.Refresh
    Else
        If (piNumTables > 0) Then
            For n = 1 To UBound(ptaTable)
'                ptaTable(n).tRecordset.Refresh
            Next n
        End If
    End If
    DoEvents

    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub

'   gsSortFields (Let)
'   PURPOSE:
'       - Sets the sortable fields of the current recordset
'   PARAMETERS:
'       - Fields (string)
'           - The sortable fields
'   RETURNS:
'       - None
Public Property Let gsSortFields(rsFields$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsSortFields (Let)"
    poError.xReset
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Can not set SortFields without an open recordset."
        Exit Property
    End If
    
    If Not xFieldExists(rsFields) Then
        poError.xSet sCurrentRoutine, "One of these fields doesn't exist in the current table: " & rsFields
        Exit Property
    End If
    
    ptaTable(piCurTable).sSortFields = rsFields
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property

'   gsFindFields (Let)
'   PURPOSE:
'       - Sets the find fields of the current recordset
'   PARAMETERS:
'       - Fields (string)
'           - The find fields
'   RETURNS:
'       - None
Public Property Let gsFindFields(rsFields$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsFindFields (Let)"
    poError.xReset
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Can not set FindFields without an open recordset."
        Exit Property
    End If
    
    If Not xFieldExists(rsFields) Then
        poError.xSet sCurrentRoutine, "One of these fields doesn't exist in the current table: " & rsFields
        Exit Property
    End If
    
    ptaTable(piCurTable).sFindFields = rsFields
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property


'   xSort
'   PURPOSE:
'       - Sorts the current recordset using the given SQL statement
'   PARAMETERS:
'       - Criteria (byvalue string)
'           - The sort criteria in standard SQL format
'   RETURNS:
'       - None
Public Sub xSort(ByVal vsCriteria$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xSort"
    poError.xReset
        
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Can not Sort without an open recordset."
        Exit Sub
    End If
        
    Dim sBuffer$, sResult$
    Dim I%, n%
    
'   Adjust the sort criteria appropriately for Access SQL by replacing
'   [Table.attribute] with Table.[attribute]

    n = InStr(vsCriteria, ".")
    Do While (n <> 0)
        sBuffer = Left$(vsCriteria, n)
        vsCriteria = Mid$(vsCriteria, n + 1)
        
        If (Left$(vsCriteria, 1) = "[") Then
'           Assume this attribute is already properly notated
            sResult = sResult & sBuffer
        Else
'           Locate the current '[' bracket
            For I = Len(sBuffer) To 1 Step -1
                If (Mid$(sBuffer, I, 1) = "[") Then
                    If (I = 1) Then
                        sResult = sResult & Mid$(sBuffer, I + 1) & "["
                    Else
                        sResult = sResult & Left$(sBuffer, I - 1) & Mid$(sBuffer, I + 1) & "["
                    End If
                    Exit For
                ElseIf (Mid$(sBuffer, I, 1) = " ") Then
'                   No bracket is to be found, this SQL statement is in error
                    poError.xSet sCurrentRoutine, "Missing proper brackets around all attributes in the SQL: ORDER BY statement."
                    Exit Sub
                End If
            Next I
        End If
        n = InStr(vsCriteria, ".")
    Loop
    ptaTable(piCurTable).sSortCriteria = sResult & vsCriteria

    xQuery
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub



'   gsSortFields (Get)
'   PURPOSE:
'       - Returns the sortable fields of the current recordset
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - The sortable fields in this recordset
Public Property Get gsSortFields$()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsSortFields (Get)"
    poError.xReset
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Can not get SortFields without an open recordset."
        Exit Property
    End If
    
    gsSortFields = ptaTable(piCurTable).sSortFields
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property

'   gsFindFields (Get)
'   PURPOSE:
'       - Returns the Findable fields of the current recordset
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - The Findable fields in this recordset
Public Property Get gsFindFields$()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsFindFields (Get)"
    poError.xReset
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Can not get FindFields without an open recordset."
        Exit Property
    End If
    
    gsFindFields = ptaTable(piCurTable).sFindFields
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property


'   gsSortCriteria (Get)
'   PURPOSE:
'       - Returns the sort criteria of the current recordset
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - The sort criteria in this recordset
Public Property Get gsSortCriteria$()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsSortCriteria (Get)"
    poError.xReset
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Can not get SortCriteria without an open recordset."
        Exit Property
    End If
    
    gsSortCriteria = ptaTable(piCurTable).sSortCriteria
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property


'   xValidate
'   PURPOSE:
'       - Validates the given input on the basis of the rules set out for
'         the given column in the recordset
'   PARAMETERS:
'       - ColumnName (string)
'           - The name of column in the recordset
'       - Value (string)
'           - The value to apply the rules to
'       - OldValue (string)
'           - The value before the edit that caused this validation
'   RETURNS:
'       - (boolean)
'           - TRUE : The rules held for the value
'           - FALSE: A rule was broken by the value
Public Function xValidate(rsColumnName$, rsValue$, rsOldValue$) As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xValidate"
    poError.xReset

    xValidate = True

    Dim sMsg$, sBuffer$
    Dim n%, m%, s%, I%
    Dim oReturn As ExtractObj: Set oReturn = New ExtractObj
    Dim bReturn As Boolean: bReturn = True
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to validate if no recordsets exist."
        Exit Function
    End If
    
    For n = 1 To ptaTable(piCurTable).iNumColumns
        If (LCase$(ptaTable(piCurTable).taColumn(n).sName) = LCase$(rsColumnName)) Then
            If (ptaTable(piCurTable).taColumn(n).iNumRules > 0) Then
                For m = 1 To UBound(ptaTable(piCurTable).taColumn(n).taRule)
                    Select Case ptaTable(piCurTable).taColumn(n).taRule(m).iRule
                    Case VRULE_ISNOTNULL
                        bReturn = (Trim$(rsValue) <> "")
                        sMsg = "Sorry, but the value given can not empty."
                    Case VRULE_ISNOTZERO
                        bReturn = ((Val(rsValue) <> 0) And (Trim$(rsValue) <> ""))
                        sMsg = "Sorry, but the value given can not be zero."
                    Case VRULE_ISNOTDECIMAL
                        bReturn = (InStr(rsValue, ".") = 0)
                        sMsg = "Sorry, but the value given can not have a decimal point."
                    Case VRULE_ISNUMERIC
                        bReturn = (IsNumeric(rsValue) And (Trim$(rsValue) <> ""))
                        sMsg = "Sorry, but the value given must be numeric."
                    Case VRULE_ISPOSITIVE
                        bReturn = (Val(rsValue) > 0)
                        sMsg = "Sorry, but the value given must be positive."
                    Case VRULE_ISNEGATIVE
                        bReturn = (Val(rsValue) < 0)
                        sMsg = "Sorry, but the value given must be negative."
                    Case VRULE_ISDATE
                        bReturn = IsDate(rsValue)
                        sMsg = "Sorry, but the value given must be a date/time."
                    Case VRULE_ISINLIST
'                       Search the column list and see if this given value exists
                        oReturn.gsValue = ptaTable(piCurTable).taColumn(n).sValidList
                        If (oReturn.giNumIndeces > 1) Then
                            For s = 1 To oReturn.giNumValues
                                If (LCase$(oReturn.gsaList(2, s)) = LCase$(rsValue)) Then Exit For
                            Next s
                        Else
                            For s = 1 To oReturn.giNumValues
                                If (LCase$(oReturn.gsaList(1, s)) = LCase$(rsValue)) Then Exit For
                            Next s
                        End If
                        
                        bReturn = (s <= oReturn.giNumValues)
                        If (s > oReturn.giNumValues) Then
                            If Not poVList Is Nothing Then
'                               Send the list to a list box
                                poVList.Clear
                                For s = 1 To oReturn.giNumValues
                                    poVList.AddItem oReturn.gsaList(1, s)
                                Next s
                                poVForm.Show 1
                            ElseIf (oReturn.giNumValues > 20) Then
                                sMsg = "Sorry, but the value given does not appear to be in the list of valid inputs."
                            ElseIf (oReturn.giNumValues <= 0) Then
                                sMsg = "Sorry, but there are no valid inputs for you to enter in this field, press ESC to exit edit."
                            Else
                                sMsg = "Sorry, but the value given must be one of the following: " & vbCrLf & "('"
                                For s = 1 To oReturn.giNumValues
                                    sMsg = sMsg & oReturn.gsaList(1, s) & "'"
                                    If (s <> oReturn.giNumValues) Then sMsg = sMsg & " '"
                                Next s
                                sMsg = sMsg & ")"
                            End If
                        End If
                    Case VRULE_ISMIN_LEN
                        bReturn = (Len(rsValue) >= ptaTable(piCurTable).taColumn(n).taRule(m).lValue)
                        sMsg = "Sorry, but the value given must be no less than " & Format$(ptaTable(piCurTable).taColumn(n).taRule(m).lValue) & " character(s) in length."
                    Case VRULE_ISMAX_LEN
                        bReturn = (Len(rsValue) <= ptaTable(piCurTable).taColumn(n).taRule(m).lValue)
                        sMsg = "Sorry, but the value given must be no greater than " & Format$(ptaTable(piCurTable).taColumn(n).taRule(m).lValue) & " character(s) in length."
                    Case VRULE_ISEXACT_LEN
                        bReturn = (Len(rsValue) = ptaTable(piCurTable).taColumn(n).taRule(m).lValue)
                        sMsg = "Sorry, but the value given must be exactly " & Format$(ptaTable(piCurTable).taColumn(n).taRule(m).lValue) & " character(s) in length."
                    Case VRULE_ISMIN_VAL
                        bReturn = (Val(rsValue) >= ptaTable(piCurTable).taColumn(n).taRule(m).lValue)
                        sMsg = "Sorry, but the value given must be no less than " & Format$(ptaTable(piCurTable).taColumn(n).taRule(m).lValue) & "."
                    Case VRULE_ISMAX_VAL
                        bReturn = (Val(rsValue) <= ptaTable(piCurTable).taColumn(n).taRule(m).lValue)
                        sMsg = "Sorry, but the value given must be no greater than " & Format$(ptaTable(piCurTable).taColumn(n).taRule(m).lValue) & "."
                    End Select
                    If Not bReturn Then
                        xValidate = False
                        If (sMsg <> "") Then MsgBox sMsg, 0, "Error in Field '" & rsColumnName & "'"
                        Exit Function
                    End If
                Next m
            End If
            
'           Check the Indeces to see if any duplicates keys have appeared
            If (rsValue <> rsOldValue) Then
                For m = 1 To ptaTable(piCurTable).iNumIndeces
                    If ptaTable(piCurTable).taIndex(m).bUnique Then
    '                   We only care about the unique indeces
                        For I = 1 To ptaTable(piCurTable).taIndex(m).iNumFields
                            If (LCase$(ptaTable(piCurTable).taIndex(m).saField(I)) = LCase$(rsColumnName)) Then
    '                           This index is of use to us
                                Exit For
                            End If
                        Next I
                        If (I <= ptaTable(piCurTable).taIndex(m).iNumFields) Then
                            sBuffer = "SELECT Count(*) AS N " & Mid$(ptaTable(piCurTable).sSQL, InStr(UCase$(ptaTable(piCurTable).sSQL), "FROM"))
                            sBuffer = sBuffer & " WHERE"
                            For I = 1 To ptaTable(piCurTable).taIndex(m).iNumFields
                                If (LCase$(ptaTable(piCurTable).taIndex(m).saField(I)) = LCase$(rsColumnName)) Then
    '                               Use the new value in place of the old
                                    If xIsString(ptaTable(piCurTable).taIndex(m).saField(I)) Then
                                        sBuffer = sBuffer & " " & ptaTable(piCurTable).taIndex(m).saField(I) & " = '"
                                        sBuffer = sBuffer & rsValue & "'"
                                    Else
                                        sBuffer = sBuffer & " " & ptaTable(piCurTable).taIndex(m).saField(I) & " = "
                                        sBuffer = sBuffer & rsValue
                                    End If
                                Else
                                    If xIsString(ptaTable(piCurTable).taIndex(m).saField(I)) Then
                                        sBuffer = sBuffer & " " & ptaTable(piCurTable).taIndex(m).saField(I) & " = '"
                                        sBuffer = sBuffer & Format$(xGet(ptaTable(piCurTable).taIndex(m).saField(I))) & "'"
                                    Else
                                        sBuffer = sBuffer & " " & ptaTable(piCurTable).taIndex(m).saField(I) & " = "
                                        sBuffer = sBuffer & Format$(xGet(ptaTable(piCurTable).taIndex(m).saField(I)))
                                    End If
                                End If
                            Next I
                            
                            ptTempSet.Close
                            Set ptTempSet = poDBase.OpenRecordset(sBuffer)
                            
                            ptTempSet.MoveFirst
                            If (ptTempSet("N") > 0) Then
    '                           Another record was found that uses the same key
                                xValidate = False
                                MsgBox "Sorry, but your change has created a duplicate key in the index '" & ptaTable(piCurTable).taIndex(m).sName & "'."
                                Exit Function
                            End If
                        End If
                    End If
                Next m
            End If
            Exit For
        End If
    Next n
    
    Exit Function
ERRORTRAP:
    Select Case Err.Number
    Case 91     ' Recordset not set
'       Caused by the first call to this routine when the recordset is closed
'       even though it was never openned before hand
    Case Else
        Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    Resume Next
End Function



'   giNumRecords (Get)
'   PURPOSE:
'       - Returns the number of records in the current recordset
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (integer)
'           - The number of records in the recordset
Public Property Get giNumRecords%()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".giNumRecords (Get)"
    poError.xReset
    
    Dim bErrorChecked As Boolean
    Dim iIndex%
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Can not get NumRecords without an open recordset."
        Exit Property
    End If
    
    xQuery "TempSet", "Count"
    
    ptTempSet.MoveFirst
    giNumRecords = Val(ptTempSet("N"))
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property


'   xCount
'   PURPOSE:
'       - Returns the number of records in the current recordset
'         that match the given criteria
'       - Note: These criteria are lumped in with the existing
'         find criteria for the current recordset
'       - Also note that the given criteria are only temporary
'   PARAMETERS:
'       - Criteria (string)
'           - SQL: WHERE statement giving the new criteria
'   RETURNS:
'       - (integer)
'           - The number of records in the recordset
Public Function xCount%(rsCriteria$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xCount"
    poError.xReset
    
    Dim bErrorChecked As Boolean
    Dim iIndex%
    Dim sOrigCriteria$
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Can not get NumRecords without an open recordset."
        Exit Function
    End If
        
'   Set up the new find criteria
    sOrigCriteria = ptaTable(piCurTable).sFindCriteria
    If (ptaTable(piCurTable).sFindCriteria <> "") Then
        ptaTable(piCurTable).sFindCriteria = ptaTable(piCurTable).sFindCriteria & " AND " & rsCriteria
    Else
        ptaTable(piCurTable).sFindCriteria = rsCriteria
    End If
        
    xQuery "TempSet", "Count"
    
    ptTempSet.MoveFirst
    xCount = Val(ptTempSet("N"))
    
'   Reset the find criteria
    ptaTable(piCurTable).sFindCriteria = sOrigCriteria
    
    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function



'   giNumKeys (Get)
'   PURPOSE:
'       - Returns the maximum number of keys that can used in ONE sort
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (integer)
'           - The maximum number of keys that can used in ONE sort
Public Property Get giNumKeys%()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".giNumKeys (Get)"
    poError.xReset
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Can not get NumKeys without an open recordset."
        Exit Property
    End If
    
    giNumKeys = ptaTable(piCurTable).iNumKeys
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property



'   giNumKeys (Let)
'   PURPOSE:
'       - Sets the maximum number of keys that can used in ONE sort
'   PARAMETERS:
'       - NumKeys (integer)
'           - The maximum number of keys that can used in ONE sort
'   RETURNS:
'       - None
Public Property Let giNumKeys(riNumKeys%)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".giNumKeys (Let)"
    poError.xReset
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Can not set NumKeys without an open recordset."
        Exit Property
    End If
    
    ptaTable(piCurTable).iNumKeys = riNumKeys
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property




'   xAddNew
'   PURPOSE:
'       - Adds a blank record to the current recordset
'   PARAMETERS:
'       - Default (optional string)
'           - Contains a listing of the default values for certain fields
'   RETURNS:
'       - None
Public Sub xAddNew(Optional rvntDefault)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xAddNew"
    poError.xReset
    Dim bErrorChecked As Boolean: bErrorChecked = False
        
    Dim n%, I%, m%
    Dim saField$()
    Dim saValue$()
    Dim iNumFields%
    Dim oReturn As ExtractObj: Set oReturn = New ExtractObj
    Dim sBuffer$
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to add a record if no recordsets are open."
        Exit Sub
    End If

    If pbDataUsed Then
        poData.Recordset.AddNew
        For n = 0 To poData.Recordset.Fields.Count - 1
            If poData.Recordset.Fields(n).Required Then
                m = xGetColumn(poData.Recordset.Fields(n).Name)
                If (m = 0) Then
'                   Set field to regular default
                    If xIsString(poData.Recordset.Fields(n)) Then
                        sBuffer = ""
                    ElseIf xIsDate(poData.Recordset.Fields(n)) Then
                        sBuffer = Format$(Now)
                    Else   ' Numeric fields
                        sBuffer = "0"
                    End If
                ElseIf (ptaTable(piCurTable).taColumn(m).sDefault = "") Then
'                   Default has not been set, ignore it
                    If xIsString(poData.Recordset.Fields(n)) Then
                        sBuffer = ""
                    ElseIf xIsDate(poData.Recordset.Fields(n)) Then
                        sBuffer = Format$(Now)
                    Else   ' Numeric fields
                        sBuffer = "0"
                    End If
                Else
                    sBuffer = ptaTable(piCurTable).taColumn(m).sDefault
                End If
                
                If xIsString(poData.Recordset.Fields(n)) Then
                    poData.Recordset.Fields(n).Value = sBuffer
                ElseIf xIsDate(poData.Recordset.Fields(n)) Then
                    poData.Recordset.Fields(n).Value = CDate(sBuffer)
                Else   ' Numeric fields
                    poData.Recordset.Fields(n).Value = Val(sBuffer)
                End If
            End If
        Next n
        
'       Set the required values
        If Not IsMissing(rvntDefault) Then
            oReturn.gsValue = CStr(rvntDefault)
            For n = 1 To oReturn.giNumValues
                If xIsString(oReturn.gsaList(1, n)) Then
                    poData.Recordset.Fields(oReturn.gsaList(1, n)).Value = oReturn.gsaList(2, n)
                ElseIf xIsDate(oReturn.gsaList(1, n)) Then
                    poData.Recordset.Fields(oReturn.gsaList(1, n)).Value = CDate(oReturn.gsaList(2, n))
                Else
                    poData.Recordset.Fields(oReturn.gsaList(1, n)).Value = Val(oReturn.gsaList(2, n))
                End If
            Next n
        End If
        
        GoTo GETFIELDS
DATAUSED:
        For n = 1 To iNumFields
            If xIsString(saField$(n)) Then
                poData.Recordset.Fields(saField$(n)).Value = saValue$(n)
            ElseIf xIsDate(saField$(n)) Then
                poData.Recordset(saField$(n)) = CDate(saValue$(n))
            Else
                poData.Recordset(saField$(n)) = Val(saValue$(n))
            End If
        Next n
    
        poData.Recordset.Update
    Else
        ptaTable(piCurTable).tRecordset.AddNew
        For n = 0 To ptaTable(piCurTable).tRecordset.Fields.Count - 1
            If ptaTable(piCurTable).tRecordset.Fields(n).Required Then
                m = xGetColumn(ptaTable(piCurTable).tRecordset.Fields(n).Name)
                If (m = 0) Then
'                   Set field to regular default
                    If xIsString(ptaTable(piCurTable).tRecordset.Fields(n)) Then
                        sBuffer = ""
                    ElseIf xIsDate(ptaTable(piCurTable).tRecordset.Fields(n)) Then
                        sBuffer = Format$(Now)
                    Else   ' Numeric fields
                        sBuffer = "0"
                    End If
                ElseIf (ptaTable(piCurTable).taColumn(m).sDefault = "") Then
'                   Default has not been set, ignore it
                    If xIsString(ptaTable(piCurTable).tRecordset.Fields(n)) Then
                        sBuffer = ""
                    ElseIf xIsDate(ptaTable(piCurTable).tRecordset.Fields(n)) Then
                        sBuffer = Format$(Now)
                    Else   ' Numeric fields
                        sBuffer = "0"
                    End If
                Else
                    sBuffer = ptaTable(piCurTable).taColumn(m).sDefault
                End If
            
                If xIsString(poData.Recordset.Fields(n)) Then
                    ptaTable(piCurTable).tRecordset.Fields(n).Value = sBuffer
                ElseIf xIsDate(poData.Recordset.Fields(n)) Then
                    ptaTable(piCurTable).tRecordset.Fields(n).Value = CDate(sBuffer)
                Else   ' Numeric fields
                    ptaTable(piCurTable).tRecordset.Fields(n).Value = Val(sBuffer)
                End If
            End If
        Next n
    
'       Set the required values
        If Not IsMissing(rvntDefault) Then
            oReturn.gsValue = CStr(rvntDefault)
            For n = 1 To oReturn.giNumValues
                If xIsString(oReturn.gsaList(1, n)) Then
                    ptaTable(piCurTable).tRecordset.Fields(oReturn.gsaList(1, n)).Value = oReturn.gsaList(2, n)
                ElseIf xIsDate(oReturn.gsaList(1, n)) Then
                    ptaTable(piCurTable).tRecordset.Fields(oReturn.gsaList(1, n)).Value = CDate(oReturn.gsaList(2, n))
                Else
                    ptaTable(piCurTable).tRecordset.Fields(oReturn.gsaList(1, n)).Value = Val(oReturn.gsaList(2, n))
                End If
            Next n
        End If
    
        GoTo GETFIELDS
DATANOTUSED:
        For n = 1 To iNumFields
            If xIsString(saField$(n)) Then
                ptaTable(piCurTable).tRecordset.Fields(saField$(n)).Value = saValue$(n)
            ElseIf xIsDate(saField$(n)) Then
                ptaTable(piCurTable).tRecordset(saField$(n)) = CDate(saValue$(n))
            Else
                ptaTable(piCurTable).tRecordset(saField$(n)) = Val(saValue$(n))
            End If
        Next n
       
        ptaTable(piCurTable).tRecordset.Update
    End If
    
    Exit Sub

GETFIELDS:
'   Determine the indeces that are unique and change the conflicting values
    For n = 1 To ptaTable(piCurTable).iNumIndeces
        If ptaTable(piCurTable).taIndex(n).bUnique Then
            If (ptaTable(piCurTable).taIndex(n).iNumFields > 0) Then
'               Only change the first field given (only one that needs to be changed)
                For I = 1 To iNumFields
                    If (ptaTable(piCurTable).taIndex(I).saField(1) = saField(I)) Then Exit For
                Next I
                If (I > iNumFields) Then
'                   Add a new index field to the listing
                    iNumFields = iNumFields + 1
                    ReDim Preserve saField$(1 To iNumFields)
                    saField$(n) = ptaTable(piCurTable).taIndex(n).saField(1)
                End If
            End If
        End If
    Next n
    
    If (iNumFields > 0) Then
        ReDim saValue$(1 To iNumFields)
    '   Find the maximum values for each of the fields
        For n = 1 To iNumFields
            xQuery "TempSet", "Max", saField$(n)
            
            ptTempSet.MoveFirst
            saValue$(n) = Format$(ptTempSet("N"))
            If (TypeName(ptTempSet("N")) = "String") Then
                saValue$(n) = saValue$(n) & "2"
            Else
                saValue$(n) = Format$(Val(saValue$(n)) + 1)
            End If
        Next n
    End If
        
    If pbDataUsed Then
        GoTo DATAUSED
    Else
        GoTo DATANOTUSED
    End If
ERRORTRAP:
    Select Case Err.Number
    Case 3022   ' Duplicate Key
        bErrorChecked = True
    Case Else
        Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    
    Resume Next
End Sub

'   xGetColumn
'   PURPOSE:
'       - Returns the index number of the column with the given Name
'   PARAMETERS:
'       - Column (string)
'           - The name of the column to find
'   RETURNS:
'       - (integer)
'           - The index number of the column
Public Function xGetColumn%(rsColumn$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xGetColumn"
    poError.xReset

    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to add a record if no recordsets are open."
        Exit Function
    End If

    Dim n%
    For n = 1 To ptaTable(piCurTable).iNumColumns
        If (LCase$(ptaTable(piCurTable).taColumn(n).sName) = LCase$(rsColumn)) Then Exit For
    Next n
    If (n > ptaTable(piCurTable).iNumColumns) Then n = 0

    xGetColumn = n

    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function


'   xDelete
'   PURPOSE:
'       - Deletes the current record from the recordset
'   PARAMETERS:
'       - None
'   RETURNS:
'       - None
Public Sub xDelete()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xDelete"
    poError.xReset
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to delete a record if no recordsets are open."
        Exit Sub
    End If

    If (MsgBox("Are you sure you wish to delete this record?", vbYesNo) = vbYes) Then
        If pbDataUsed Then
            poData.Recordset.Delete
        Else
            ptaTable(piCurTable).tRecordset.Delete
        End If
    End If
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub



'   xMassDelete
'   PURPOSE:
'       - Deletes all records where the given parameters are met in current table
'   PARAMETERS:
'       - Params (string)
'           - A SQL: WHERE statement containing the required parameters to be met
'       - TableName (optional string)
'           - The name of the table to delete the records from
'           - This is only relevant when a JOINed recordset is being used
'   RETURNS:
'       - None
Public Sub xMassDelete(rsParams$, Optional rvntTableName)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xMassDelete"
    poError.xReset
    
    Dim sBuffer$
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to delete records if no recordsets are open."
        Exit Sub
    End If

'   Create a query to handle the deletion
    If IsMissing(rvntTableName) Then
        sBuffer = "DELETE * " & Mid$(ptaTable(piCurTable).sSQL, InStr(UCase$(ptaTable(piCurTable).sSQL), "FROM"))
    Else
        sBuffer = "DELETE " & CStr(rvntTableName) & ".* " & Mid$(ptaTable(piCurTable).sSQL, InStr(UCase$(ptaTable(piCurTable).sSQL), "FROM"))
    End If
        
    If ((ptaTable(piCurTable).sFindCriteria <> "") And (ptaTable(piCurTable).sRequiredFind <> "")) Then
        sBuffer = sBuffer & " WHERE " & ptaTable(piCurTable).sFindCriteria & " AND " & ptaTable(piCurTable).sRequiredFind
    ElseIf (ptaTable(piCurTable).sFindCriteria <> "") Then
        sBuffer = sBuffer & " WHERE " & ptaTable(piCurTable).sFindCriteria
    ElseIf (ptaTable(piCurTable).sRequiredFind <> "") Then
        sBuffer = sBuffer & " WHERE " & ptaTable(piCurTable).sRequiredFind
    Else
    End If
        
    If (rsParams <> "") Then
        If (InStr(sBuffer, "WHERE") <> 0) Then
            sBuffer = sBuffer & " AND " & rsParams
        Else
            sBuffer = sBuffer & " WHERE " & rsParams
        End If
    End If
    
    If pbDataUsed Then
        poData.Database.Execute sBuffer
    Else
        poDBase.Execute sBuffer
    End If
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub



'   xMassUpdate
'   PURPOSE:
'       - Updates all records where the given parameters are met in current table
'   PARAMETERS:
'       - Params (string)
'           - A SQL: SET statement containing the data to update
'       - Criteria (optional string)
'           - The SQL: WHERE clause of the update
'           - If left out the old find criteria is used instead
'   RETURNS:
'       - None
Public Sub xMassUpdate(rsParams$, Optional rvntCriteria)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xMassUpdate"
    poError.xReset
    
    Dim sBuffer$
    Dim sCriteria$
        
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to update records if no recordsets are open."
        Exit Sub
    End If

    If IsMissing(rvntCriteria) Then
        sCriteria = ptaTable(piCurTable).sFindCriteria
    Else
        sCriteria = CStr(rvntCriteria)
    End If

'   Create a query to handle the update
    sBuffer = "UPDATE " & Mid$(ptaTable(piCurTable).sSQL, InStr(UCase$(ptaTable(piCurTable).sSQL), "FROM") + Len("FROM"))
        
    sBuffer = sBuffer & " SET " & rsParams
    
    If ((sCriteria <> "") And (ptaTable(piCurTable).sRequiredFind <> "")) Then
        sBuffer = sBuffer & " WHERE " & sCriteria & " AND " & ptaTable(piCurTable).sRequiredFind
    ElseIf (sCriteria <> "") Then
        sBuffer = sBuffer & " WHERE " & sCriteria
    ElseIf (ptaTable(piCurTable).sRequiredFind <> "") Then
        sBuffer = sBuffer & " WHERE " & ptaTable(piCurTable).sRequiredFind
    End If
        
    If pbDataUsed Then
        poData.Database.Execute sBuffer
    Else
        poDBase.Execute sBuffer
    End If
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub




'   xMoveNext
'   PURPOSE:
'       - Move to the next record in the recordset
'       - If none, then return FALSE
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (boolean)
'           - TRUE : A record was found
'           - FALSE: No record was found
Public Function xMoveNext() As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xMoveNext"
    poError.xReset

    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to move to a record if no recordsets are open."
        Exit Function
    End If
    
    ptaTable(piCurTable).tRecordset.MoveNext
    If ptaTable(piCurTable).tRecordset.EOF Then
        ptaTable(piCurTable).tRecordset.MoveLast
        xMoveNext = False
    Else
        xMoveNext = True
    End If

    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

'   xMoveFirst
'   PURPOSE:
'       - Move to the first record in the recordset
'       - If none, then return FALSE
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (boolean)
'           - TRUE : A record was found
'           - FALSE: No record was found
Public Function xMoveFirst() As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xMoveFirst"
    poError.xReset
    Dim bErrorChecked As Boolean: bErrorChecked = False

    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to move to a record if no recordsets are open."
        Exit Function
    End If
    
    ptaTable(piCurTable).tRecordset.MoveFirst
    xMoveFirst = Not bErrorChecked

    Exit Function
ERRORTRAP:
    Select Case Err.Number
    Case 3021
        bErrorChecked = True
    Case Else
        Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    Resume Next
End Function


'   xMoveLast
'   PURPOSE:
'       - Move to the last record in the recordset
'       - If none, then return FALSE
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (boolean)
'           - TRUE : A record was found
'           - FALSE: No record was found
Public Function xMoveLast() As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xMoveLast"
    poError.xReset
    Dim bErrorChecked As Boolean: bErrorChecked = False

    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to move to a record if no recordsets are open."
        Exit Function
    End If
    
    ptaTable(piCurTable).tRecordset.MoveLast
    xMoveLast = Not bErrorChecked

    Exit Function
ERRORTRAP:
    Select Case Err.Number
    Case 3021
        bErrorChecked = True
    Case Else
        Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    Resume Next
End Function



'   xMovePrev
'   PURPOSE:
'       - Move to the previous record in the recordset
'       - If none, then return FALSE
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (boolean)
'           - TRUE : A record was found
'           - FALSE: No record was found
Public Function xMovePrev() As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xMovePrev"
    poError.xReset

    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to move to a record if no recordsets are open."
        Exit Function
    End If
    
    ptaTable(piCurTable).tRecordset.MovePrevious
    If ptaTable(piCurTable).tRecordset.BOF Then
        ptaTable(piCurTable).tRecordset.MoveFirst
        xMovePrev = False
    Else
        xMovePrev = True
    End If
    
    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function


'   xGetList
'   PURPOSE:
'       - Returns an extractable listing where the given keys and values are stored
'   PARAMETERS:
'       - Description (string)
'           - The field to act as the description
'       - Value (string)
'           - The field to act as the value
'   RETURNS:
'       - (String)
'           - An extractable listing of the keys and values
Public Function xGetList$(rsDescription$, rsValue$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xGetList"
    poError.xReset
    Dim bErrorChecked As Boolean: bErrorChecked = False
    
    Dim sBuffer$
    
    If (piNumTables <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to get a listing if no recordsets are open."
        Exit Function
    End If

'   Determine if the fields given actually exist in the current table
    If Not xFieldExists(rsDescription) Then
        poError.xSet sCurrentRoutine, "The field '" & rsDescription & "' doesn't exist in the current table."
        Exit Function
    ElseIf Not xFieldExists(rsValue) Then
        poError.xSet sCurrentRoutine, "The field '" & rsValue & "' doesn't exist in the current table."
        Exit Function
    End If

'   Call xQuery
    xQuery "UseTempSet"
    
    ptTempSet.MoveFirst
    If bErrorChecked Then
        sBuffer = ""
    Else
        sBuffer = "(2) '|' " & ptTempSet(rsDescription) & " | " & ptTempSet(rsValue)
        ptTempSet.MoveNext
        Do While (Not ptTempSet.EOF)
            sBuffer = sBuffer & " | " & ptTempSet(rsDescription) & " | " & ptTempSet(rsValue)
            ptTempSet.MoveNext
        Loop
    End If
    
    xGetList = sBuffer

    Exit Function
ERRORTRAP:
    Select Case Err.Number
    Case 3021   'No current record
        bErrorChecked = True
    Case Else
        Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    Resume Next
End Function


'   xFieldExists
'   PURPOSE:
'       - Determines whether the given field(s) exists in the current table
'   PARAMETERS:
'       - Fields (string)
'           - An extractable listing of the field names
'   RETURNS:
'       - (boolean)
'           - TRUE : All fields exist
'           - FALSE: At least on of the fields given doesn't exist
Public Function xFieldExists(rsFields$) As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xFieldExists"
    poError.xReset
    Dim bErrorChecked As Boolean: bErrorChecked = False

    Dim oReturn As ExtractObj: Set oReturn = New ExtractObj
    Dim n%, vntTemp As Variant

'   Check to see if all the fields given actually exist in the current recordset
    oReturn.gsValue = rsFields
    If pbDataUsed Then
        For n = 1 To oReturn.giNumValues
    '       An error should occur if the field is not found
            vntTemp = poData.Recordset.Fields(oReturn.gsaList(1, n)).Type
        Next n
    Else
        For n = 1 To oReturn.giNumValues
    '       An error should occur if the field is not found
            vntTemp = ptaTable(piCurTable).tRecordset.Fields(oReturn.gsaList(1, n)).Type
        Next n
    End If

    xFieldExists = Not bErrorChecked

    Exit Function
ERRORTRAP:
    Select Case Err.Number
    Case 3265   ' Field doesn't exist
        bErrorChecked = True
    Case Else
        Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    Resume Next
End Function

'   xMax
'   PURPOSE:
'       - Returns the maximum value of the given field in the current recordset
'   PARAMETERS:
'       - Field (string)
'           - The name of the field to return the maximum value for
'   RETURNS:
'       - (string)
'           - The maximum value
Public Function xMax$(rsField$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xMax"
    poError.xReset

    If (giNumRecords > 0) Then
        xQuery "TempSet", "Max", rsField
    
        ptTempSet.MoveFirst
        xMax = CStr(ptTempSet("N"))
    Else
        xMax = ""
    End If

    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

'   xMin
'   PURPOSE:
'       - Returns the minimum value of the given field in the current recordset
'   PARAMETERS:
'       - Field (string)
'           - The name of the field to return the minimum value for
'   RETURNS:
'       - (string)
'           - The minimum value
Public Function xMin$(rsField$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xMin"
    poError.xReset

    If (giNumRecords > 0) Then
        xQuery "TempSet", "Min", rsField
    
        ptTempSet.MoveFirst
        xMin = CStr(ptTempSet("N"))
    Else
        xMin = ""
    End If

    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function


'   xUpdate
'   PURPOSE:
'       - Store any unsaved data in the current record
'   PARAMETERS:
'       - None
'   RETURNS:
'       - None
Public Sub xUpdate()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xUpdate"
    poError.xReset

    If pbDataUsed Then poData.UpdateRecord

    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub



'   goVForm (Set)
'   PURPOSE:
'       - Sets the form to show the valid values list on
'       - Also makes certain that a list box exists in the form
'   PARAMETERS:
'       - Form (form control)
'           - The form to check
'   RETURNS:
'       - None
Public Property Set goVForm(roForm As Form)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".goVForm (Set)"
    poError.xReset

    Dim n%
    
    For n = 0 To roForm.Count - 1
        If (TypeOf roForm.Controls(n) Is ListBox) Or (TypeOf roForm.Controls(n) Is ComboBox) Then
            Set poVList = roForm.Controls(n)
            Exit For
        End If
    Next n
    If (n > roForm.Count - 1) Then
        poError.xSet sCurrentRoutine, "The form given doesn't have a listbox or combobox on it."
        Exit Property
    End If

    Set poVForm = roForm
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property
