VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ErrorObj"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'   CONSTANTS
'----------------------------------------------------------------------------------

Private Const OBJ_NAME = "ErrorObj"

'----------------------------------------------------------------------------------
'   OBJECTS
'----------------------------------------------------------------------------------

Private poFile As New FileObj       ' A private file object used by error
Private poIni As New IniObj         ' Reference to an INI object

'----------------------------------------------------------------------------------
'   STRUCTURES
'----------------------------------------------------------------------------------

' None

'----------------------------------------------------------------------------------
'   VARIABLES
'----------------------------------------------------------------------------------

Private psFileName$                 ' Contains the filename of the error log
Private pbOnErrorAbort As Boolean   ' On an error abort the program (default TRUE)
Private psErrorMessage As String    ' The Error message
Private psErrorRoutine As String    ' The routine where the error occurred

'   gbOnErrorAbort (Let)
'   PURPOSE:
'       - Sets the 'On Error Abort' choice
'       - This affects whether the program terminates when a fatal error occurs
'   PARAMETERS:
'       - Value (boolean)
'           - The value to set OnErrorAbort
'   RETURNS:
'       - None
Public Property Let gbOnErrorAbort(rbValue As Boolean)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsOnErrorAbort"
    
    pbOnErrorAbort = rbValue

    Exit Property
ERRORTRAP:
    Call xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property


'   xSet
'   PURPOSE:
'       - Captures fatal errors and exits the program if FatalAbort = True
'       - Otherwise it just logs the problem and
'   PARAMETERS:
'       - CurrentRoutine (string)
'           - The routine the error occurred in
'       - Error (string)
'           - The error message
'   RETURNS:
'       - None
Public Sub xSet(rsCurrentRoutine$, rsError$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xSetError"

    If pbOnErrorAbort Then xEnd rsCurrentRoutine, rsError
    psErrorRoutine = rsCurrentRoutine
    psErrorMessage = rsError

    Exit Sub
ERRORTRAP:
    Call xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   xEnd
'   PURPOSE:
'       - This routine is called when a fatal user-defined error occurs
'       - It will close down the system and declare an error (if given one)
'   PARAMETERS:
'       - CurrentRoutine (optional string)
'           - The Routine from which this routine was called
'       - Error (optional string)
'           - an error message
'   RETURNS:
'       - None
Public Sub xEnd(Optional rvntCurrentRoutine, Optional rvntError)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xEnd"
    
    Dim sMsg$
    If Not IsMissing(rvntError) Then
        If (rvntError <> "") Then
            sMsg = "Program has halted prematurely...  "
            MsgBox sMsg & CStr(rvntError)
            If IsMissing(rvntCurrentRoutine) Then
                Call xLog(-1, "(No routine name given)", sMsg & CStr(rvntError))
            Else
                Call xLog(-1, CStr(rvntCurrentRoutine), sMsg & CStr(rvntError))
            End If
        End If
    End If
    
    End
    
    Exit Sub
ERRORTRAP:
    Call xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub





'   xReset
'   PURPOSE:
'       - Resets the error handler
'   PARAMETERS:
'       - None
'   RETURNS:
'       - None
Public Sub xReset()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xResetError"

    psErrorRoutine = ""
    psErrorMessage = ""

    Exit Sub
ERRORTRAP:
    Call xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub



'   xLog
'   PURPOSE:
'       - Handles the storing of errors as they occur for later review
'   PARAMETERS:
'       - ErrNumber (integer)
'           - The error code generated (may be -1: User defined)
'       - CurrentRoutine (string)
'           - The name of the routine the error occurred within
'       - Description (string)
'           - The error message that goes with the error code
'   RETURNS:
'       - None
Public Sub xLog(riErrNumber%, rsCurrentRoutine$, rsDescription$)
    Dim sText$
    Dim bShowMsg As Boolean
    
    If (riErrNumber = -1) Then
'       Personal code error
        sText$ = "Personal Error was generated by " & rsCurrentRoutine
    Else
'       RunTime error
        sText$ = "Runtime Error #" & Format$(riErrNumber) & " was generated by " & rsCurrentRoutine
    End If
    
'   Give special messages for special errors
    bShowMsg = False
    Select Case riErrNumber
    Case 7  'Out of memory
        MsgBox "Windows has run out of free memory.  Consider closing other applications or restarting Windows all together."
    Case 3026   'Out of diskspace
        MsgBox "This application has run out of free diskspace.  Consider freeing up diskspace before rerunning this program."
    Case Else
        bShowMsg = True
    End Select
    
    If Not poFile.xFileExists(psFileName) Then
'       File doesn't exist, create a header for the new file
        poFile.xInitialize psFileName, "w"
        
        poFile.xSet "Log Created at", "CR"
        poFile.xSet "Date: " & Format$(Date), "CR"
        poFile.xSet "Time: " & Format$(Time), "CR"
        poFile.xSet "--------------", "CR"
    End If
    
'   Append new error to the ErrorLog
    poFile.xInitialize psFileName, "a"
    
    poFile.xSet sText$, "CR"
    poFile.xSet vbTab & rsDescription

    If bShowMsg Then
        If (riErrNumber <> -1) Then If (MsgBox(sText$ & vbCrLf & rsDescription & vbCrLf & "Do you want to quit now?", 4, "A fatal error occurred") = 6) Then End
    Else
        End
    End If
End Sub




Private Sub Class_Initialize()
    Dim iCounter%
    
'   Determine the name of the error file to use in this session
    If poFile.xFileExists(App.Path & "\error.log") Then
'       An old error log exists, move to ErrorLog directory
        If Not poFile.xDirExists(App.Path & "\ErrorLog") Then MkDir App.Path & "\ErrorLog"
        
'       Use the counter stored in the INI file
        iCounter = Val(poIni.xGet("ErrorLogging", "Counter", "0"))
        psFileName = App.Path & "\ErrorLog\error" & Format$(iCounter, "000") & ".log"
        
'       Remove old ErrorLog
        If poFile.xFileExists(psFileName) Then Kill psFileName
        
        iCounter = iCounter + 1
        If (iCounter > 999) Then iCounter = 0
        poIni.xSet "ErrorLogging", "Counter", Format$(iCounter%)
        
'       Copy newer log into ErrorLog directory, and then kill other copy
        FileCopy App.Path & "\error.log", psFileName
        Kill App.Path & "\error.log"
    End If
    psFileName = App.Path & "\error.log"

    pbOnErrorAbort = True
End Sub


'   gbIsError
'   PURPOSE:
'       - Returns whether a fatal error occurred within the routine
'       - Usually called after a routine finishs
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (boolean)
'           - TRUE : A fatal error occurred
'           - FALSE: No fatal error occurred
Public Property Get gbIsError() As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gbIsError"

    gbIsError = (psErrorMessage <> "")
    
    Exit Property
ERRORTRAP:
    Call xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property



Private Sub Class_Terminate()
    Set poIni = Nothing
    Set poFile = Nothing
End Sub


