VERSION 1.0 CLASS
BEGIN
  MultiUse = 0   'False
END
Attribute VB_Name = "FileObj"
Attribute VB_Creatable = True
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'   CONSTANTS
'----------------------------------------------------------------------------------

Private Const OBJ_NAME = "FileObj"

'----------------------------------------------------------------------------------
'   OBJECTS
'----------------------------------------------------------------------------------

' None

'----------------------------------------------------------------------------------
'   STRUCTURES
'----------------------------------------------------------------------------------

' None

'----------------------------------------------------------------------------------
'   VARIABLES
'----------------------------------------------------------------------------------

Private piFile As Integer            ' The file identifier
Private psFileName As String         ' The FileName
Private psParams As String           ' The params the file was opened with
'   FileExists
'   PURPOSE:
'       - returns whether the given file name cooresponds to an existing file
'   PARAMETERS:
'       - FileName (string)
'           - The name of the file to check for
'   RETURN:
'       - None
Public Function xFileExists(ByVal vsFileName$) As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xFileExists"
    
    xFileExists = True
    
    Dim iX%
    iX = FreeFile
    Open vsFileName$ For Input As #iX
    Close #iX

    Exit Function
ERRORTRAP:
    xFileExists = False
    Resume Next
End Function

'   xDirExists
'   PURPOSE:
'       - Determines if a directory already exists by creating it and seeing
'         if an error occurs
'   PARAMETERS:
'       - Path (string)
'           - The directory to check
'   RETURNS:
'       - (boolean)
'           - TRUE : The directory exists
'           - FALSE: The directory doesn't exist
Public Function xDirExists(rsPath$) As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xDirExists"
    
    xDirExists = True
    
    ChDir rsPath

    Exit Function
ERRORTRAP:
    xDirExists = False
    Resume Next
End Function



'   Initialize
'   PURPOSE:
'       - Initializes the File Class, closes a previous use (if any)
'   PARAMETERS:
'       - FileName (string)
'           - the name of the file to open
'       - Parameters (string)
'           - the options to open the file with
'           - r: read
'           - w: write
'           - a: append
'   RETURN:
'       (boolean)
'           - TRUE : File was openned for given parameters
'           - FALSE: File could not be openned
Public Function xInitialize(ByVal vsFileName$, ByVal vsParameters$) As Boolean
    xInitialize = True
    
    If (piFile <> 0) Then
'       Close previously opened file before continuing
        Close #piFile
    End If

'   Open the given file (FreeFile returns an unused File descriptor)
    piFile = FreeFile
    psFileName = vsFileName
    
    If (InStr(vsParameters, "a") <> 0) Then
'       Open this file for appending
        If Not xFileExists(vsFileName) Then
            xInitialize = False
            Exit Function
        End If
    
        Open psFileName For Append As #piFile
    ElseIf (InStr(vsParameters, "r") <> 0) Then
'       Open the file for reading (so check if it exists)
        If Not xFileExists(vsFileName) Then
            xInitialize = False
            Exit Function
        End If
        
        If (InStr(vsParameters, "w") <> 0) Then
'           Open the file for read/write access
            Open psFileName For Random Access Read Write As #piFile
        Else
            Open psFileName For Input As #piFile
        End If
    ElseIf (InStr(vsParameters, "w") <> 0) Then
        Open psFileName For Output As #piFile
    Else
        xInitialize = False
    End If
    
    psParams = vsParameters
End Function




'   Class_Terminate
'   PURPOSE:
'       - Destroys the File Class
'       - Closes any open files
'   PARAMETERS:
'       - None
'   RETURN:
'       - None
Private Sub Class_Terminate()
    If (piFile <> 0) Then Close #piFile
End Sub



'   EOF
'   PURPOSE:
'       - Returns whether the end of file has been reached or not
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (boolean)
'           - TRUE : EOF has been reached
'           - FALSE: EOF has NOT been reached
Public Property Get gbEOF() As Boolean
    gbEOF = EOF(piFile)
End Property



'   Get
'   PURPOSE:
'       - Returns a line of data from the open file
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - A line of data
Public Function xGet$()
    Dim sBuffer$
    
    If (InStr(psParams, "r") = 0) Then
        MsgBox "ERROR: File Class attempted to read from a write only file."
        End
    End If
    
    xGet = ""
    If EOF(piFile) Then Exit Function
    
    Input #piFile, sBuffer
    xGet = sBuffer
End Function

'   Set
'   PURPOSE:
'       - Prints a given line of text to the open file
'   PARAMETERS:
'       - Buffer (string)
'           - A line of data
'       - CR (optional boolean)
'           - If given the line is terminated with a carriage return
'   RETURNS:
'       - None
Public Sub xSet(ByVal vsBuffer$, Optional rvntCR)
    If (InStr(psParams, "w") = 0) And (InStr(psParams, "a") = 0) Then
        MsgBox "ERROR: File Class attempted to write to a read only file."
        End
    End If
    
    If IsMissing(rvntCR) Then
        Print #piFile, vsBuffer;
    Else
        Print #piFile, vsBuffer
    End If
End Sub

