VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "FormObj"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'   BINDING RULES
'----------------------------------------------------------------------------------

'   Object Initialization
'       <FormObj object>.xInitialize <Form control>'
'       <FormObj object>.xCenter
'
'   poForm (Form control)
'       Resize
'           <FormObj object>.xResize

'----------------------------------------------------------------------------------
'   CONSTANTS
'----------------------------------------------------------------------------------

Private Const OBJ_NAME = "FormObj"

'----------------------------------------------------------------------------------
'   OBJECTS
'----------------------------------------------------------------------------------

Private poForm As Form                      ' The bound form
Private poError As New ErrorObj             ' The error handling routines
Private poIni As New IniObj                 ' The Ini file handler

'----------------------------------------------------------------------------------
'   STRUCTURES
'----------------------------------------------------------------------------------

Private Type StyleType
    iStyle As Integer                       ' The style of resize
    tAlign As Control                       ' The control to align to (optional)
    iAlignValue As Integer                  ' The distance from the Align control
    fPercent As Single                      ' The percent of the resize change to use
End Type

Private Type ResizeObjectType
    sName As String                         ' The name of the control to resize
    iIndex As Integer                       ' The index (if any) of the control
    tObject As Control                      ' The control to resize (if given)
    taStyle() As StyleType                  ' The styles themselves
    iNumStyles As Integer                   ' The number of resizing styles
    iCurStyle As Integer                    ' The current style selected
End Type

'----------------------------------------------------------------------------------
'   VARIABLES
'----------------------------------------------------------------------------------

Private plScaleWidth&                       ' The scale width to hold the form at
Private plOldWidth&                         ' The previous width
Private glMinWidth&                         ' The minimum width

Private plScaleHeight&                      ' The scale height to hold the form at
Private plOldHeight&                        ' The previous height
Private glMinHeight&                        ' The minimum height

Private pbResizing As Boolean               ' Resize routine is running

Private piNumObjects As Integer              ' The number of objects to Resize
Private ptaResizeObject() As ResizeObjectType ' The objects to resize
Private piCurResizeObject As Integer         ' The current resize object

Private pbInitialized As Boolean            ' Whether xInitialize has been run

'   glScaleWidth (Let)
'   PURPOSE:
'       - Sets the Scale and Old Width of the bound form
'   PARAMETERS:
'       - Width (long)
'           - The width to set
'   RETURNS:
'       - None
Public Property Let glScaleWidth(rlWidth&)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".glScaleWidth (Let)"
    poError.xReset
    
    plScaleWidth = rlWidth
    plOldWidth = rlWidth

    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property


'   glScaleHeight (Let)
'   PURPOSE:
'       - Sets the Scale and Old Height of the bound form
'   PARAMETERS:
'       - Height (long)
'           - The Height to set
'   RETURNS:
'       - None
Public Property Let glScaleHeight(rlHeight&)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".glScaleHeight (Let)"
    poError.xReset
    
    plScaleHeight = rlHeight
    plOldHeight = rlHeight

    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property



'   gfStylePercent (Let)
'   PURPOSE:
'       - Sets the Percent of a resize change to use with the current style
'   PARAMETERS:
'       - Percent (single)
'           - The percentage (0.0 to 1.0)
'   RETURNS:
'       - None
Public Property Let gfStylePercent(rfPercent!)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".gfStylePercent (Let)"
    poError.xReset
    
    If (piNumObjects <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to change percent if no Resize Object exist."
        Exit Property
    End If
    
    If (ptaResizeObject(piCurResizeObject).iNumStyles <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to change percent if no styles exist."
        Exit Property
    End If
    
    Dim n%
    
    n = ptaResizeObject(piCurResizeObject).iCurStyle
    ptaResizeObject(piCurResizeObject).taStyle(n).fPercent = rfPercent

    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property




'   xCenter
'   PURPOSE:
'       - Centers the form on the screen
'       - Or loads up previous values from the INI file and uses those
'   PARAMETERS:
'       - None
'   RETURNS:
'       - None
Public Sub xCenter()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xCenter"
    poError.xReset
         
    Dim oReturn As ExtractObj: Set oReturn = New ExtractObj
    
'   Load any older values stored in the INI file
    oReturn.gsValue = poIni.xGet("Forms", poForm.Name, "")
    If (oReturn.giNumValues > 0) Then
        poForm.Top = oReturn.gsaList(2, 1)
        poForm.Left = oReturn.gsaList(2, 2)
        poForm.Width = oReturn.gsaList(2, 3)
        poForm.Height = oReturn.gsaList(2, 4)
    
'       Determine if the form is not on the screen, due to changes in the INI file
'       directly or changes to the screen resolution
    
'       Screen is too big for the current resolution
        If (poForm.Width > Screen.Width) Then poForm.Width = Screen.Width
        If (poForm.Height > Screen.Height) Then poForm.Height = Screen.Height
        
'       Out of view to the Left or Top of screen
        If (poForm.Left + poForm.Width < 0) Or (poForm.Top + poForm.Height < 0) Then
            poForm.Move (Screen.Width - poForm.Width) / 2, (Screen.Height - poForm.Height) / 2
        End If
        
'       Out of view to the Right or Bottom of screen
        If (poForm.Left > Screen.Width) Or (poForm.Top > Screen.Height) Then
            poForm.Move (Screen.Width - poForm.Width) / 2, (Screen.Height - poForm.Height) / 2
        End If
    Else
'       Center the form on the screen
        poForm.Move (Screen.Width - poForm.Width) / 2, (Screen.Height - poForm.Height) / 2
    End If

    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub

'   xInitialize
'   PURPOSE:
'       - Initializes the class
'       - Binds the form to work with
'   PARAMETERS:
'       - Form (form control)
'           - The form to bind
'   RETURNS:
'       - None
Public Sub xInitialize(roForm As Form)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = roForm.Name & "." & OBJ_NAME & ".xInitialize"
    poError.xReset

    If pbInitialized Then
        poError.xSet sCurrentRoutine, "You can not initialize object twice."
        Exit Sub
    End If
    pbInitialized = True
        
    Set poForm = roForm

    plOldWidth = poForm.ScaleWidth
    plOldHeight = poForm.ScaleHeight
    glMinWidth = poForm.ScaleWidth
    glMinHeight = poForm.ScaleHeight

    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   xResizeObjects
'   PURPOSE:
'       - Resize to the given ScaleWidth and ScaleHeight
'       - Resizes the objects on the screen to fit the new size
'   PARAMETERS:
'       - None
'   RETURNS:
'       - None
Public Sub xResize()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xResize"
    poError.xReset

    If pbResizing Then Exit Sub
    pbResizing = True

    Dim n%, m%, s%
    Dim lWidthChg&: lWidthChg = 0
    Dim lHeightChg&: lHeightChg = 0

    If (plScaleWidth > 0) Then
        poForm.Width = (poForm.Width - poForm.ScaleWidth) + plScaleWidth
    Else
        If (poForm.ScaleWidth < glMinWidth) Then poForm.Width = (poForm.Width - poForm.ScaleWidth) + glMinWidth
        lWidthChg = poForm.ScaleWidth - plOldWidth
                
        plOldWidth = poForm.ScaleWidth
    End If
    If (plScaleHeight > 0) Then
        poForm.Height = (poForm.Height - poForm.ScaleHeight) + plScaleHeight
    Else
        If (poForm.ScaleHeight < glMinHeight) Then poForm.Height = (poForm.Height - poForm.ScaleHeight) + glMinHeight
        lHeightChg = poForm.ScaleHeight - plOldHeight
        
        plOldHeight = poForm.ScaleHeight
    End If

'   Move the rest of the controls given
    For n = 1 To piNumObjects
        For m = 1 To ptaResizeObject(n).iNumStyles
            If (ptaResizeObject(n).iIndex <> FINDEX_ALL) Then
'               The given control need not be located, it was found when added to list
                Select Case ptaResizeObject(n).taStyle(m).iStyle
                Case FRESIZE_X_STRETCH
                    ptaResizeObject(n).tObject.Width = ptaResizeObject(n).tObject.Width + lWidthChg * ptaResizeObject(n).taStyle(m).fPercent
                Case FRESIZE_X_MOVE
                    ptaResizeObject(n).tObject.Left = ptaResizeObject(n).tObject.Left + lWidthChg * ptaResizeObject(n).taStyle(m).fPercent
                Case FRESIZE_X_CENTER
                    ptaResizeObject(n).tObject.Left = (poForm.ScaleWidth - ptaResizeObject(n).tObject.Width) / 2
                Case FRESIZE_Y_STRETCH
                    ptaResizeObject(n).tObject.Height = ptaResizeObject(n).tObject.Height + lHeightChg * ptaResizeObject(n).taStyle(m).fPercent
                Case FRESIZE_Y_MOVE
                    ptaResizeObject(n).tObject.Top = ptaResizeObject(n).tObject.Top + lHeightChg * ptaResizeObject(n).taStyle(m).fPercent
                Case FRESIZE_Y_CENTER
                    ptaResizeObject(n).tObject.Top = (poForm.ScaleHeight - ptaResizeObject(n).tObject.Height) / 2
                End Select
            Else
'               Search for all controls that have the same name as given
                For s = 0 To poForm.Controls.Count - 1
                    If (LCase$(poForm.Controls(s).Name) = LCase$(ptaResizeObject(n).sName)) Then
                        Select Case ptaResizeObject(n).taStyle(m).iStyle
                        Case FRESIZE_X_STRETCH
                            poForm.Controls(s).Width = poForm.Controls(s).Width + lWidthChg * ptaResizeObject(n).taStyle(m).fPercent
                        Case FRESIZE_X_MOVE
                            poForm.Controls(s).Left = poForm.Controls(s).Left + lWidthChg * ptaResizeObject(n).taStyle(m).fPercent
                        Case FRESIZE_X_CENTER
                            poForm.Controls(s).Left = (poForm.ScaleWidth - poForm.Controls(s).Width) / 2
                        Case FRESIZE_Y_STRETCH
                            poForm.Controls(s).Height = poForm.Controls(s).Height + lHeightChg * ptaResizeObject(n).taStyle(m).fPercent
                        Case FRESIZE_Y_MOVE
                            poForm.Controls(s).Top = poForm.Controls(s).Top + lHeightChg * ptaResizeObject(n).taStyle(m).fPercent
                        Case FRESIZE_Y_CENTER
                            poForm.Controls(s).Top = (poForm.ScaleHeight - poForm.Controls(s).Height) / 2
                        End Select
                    End If
                Next s
            End If
        Next m
    Next n

'   Perform alignment changes (after the objects have moved, changed size, etc.)
    For n = 1 To piNumObjects
        For m = 1 To ptaResizeObject(n).iNumStyles
            If (ptaResizeObject(n).iIndex <> FINDEX_ALL) Then
'               The given control need not be located, it was found when added to list
                Select Case ptaResizeObject(n).taStyle(m).iStyle
                Case FRESIZE_ALIGN_ABOVE
                    ptaResizeObject(n).tObject.Top = ptaResizeObject(n).taStyle(m).tAlign.Top - ptaResizeObject(n).tObject.Height + ptaResizeObject(n).taStyle(m).iAlignValue
                Case FRESIZE_ALIGN_BELOW
                    ptaResizeObject(n).tObject.Top = ptaResizeObject(n).taStyle(m).tAlign.Top + ptaResizeObject(n).taStyle(m).tAlign.Height + ptaResizeObject(n).taStyle(m).iAlignValue
                Case FRESIZE_ALIGN_LEFT
                    ptaResizeObject(n).tObject.Left = ptaResizeObject(n).taStyle(m).tAlign.Left - ptaResizeObject(n).tObject.Width + ptaResizeObject(n).taStyle(m).iAlignValue
                Case FRESIZE_ALIGN_RIGHT
                    ptaResizeObject(n).tObject.Left = ptaResizeObject(n).taStyle(m).tAlign.Left + ptaResizeObject(n).taStyle(m).tAlign.Width + ptaResizeObject(n).taStyle(m).iAlignValue
                End Select
            Else
'               Search for all controls that have the same name as given
                For s = 0 To poForm.Controls.Count - 1
                    If (LCase$(poForm.Controls(s).Name) = LCase$(ptaResizeObject(n).sName)) Then
                        Select Case ptaResizeObject(n).taStyle(m).iStyle
                        Case FRESIZE_ALIGN_ABOVE
                            poForm.Controls(s).Top = ptaResizeObject(n).taStyle(m).tAlign.Top - poForm.Controls(s).Height + ptaResizeObject(n).taStyle(m).iAlignValue
                        Case FRESIZE_ALIGN_BELOW
                            poForm.Controls(s).Top = ptaResizeObject(n).taStyle(m).tAlign.Top + ptaResizeObject(n).taStyle(m).tAlign.Height + ptaResizeObject(n).taStyle(m).iAlignValue
                        Case FRESIZE_ALIGN_LEFT
                            poForm.Controls(s).Left = ptaResizeObject(n).taStyle(m).tAlign.Left - poForm.Controls(s).Width + ptaResizeObject(n).taStyle(m).iAlignValue
                        Case FRESIZE_ALIGN_RIGHT
                            poForm.Controls(s).Left = ptaResizeObject(n).taStyle(m).tAlign.Left + ptaResizeObject(n).taStyle(m).tAlign.Width + ptaResizeObject(n).taStyle(m).iAlignValue
                        End Select
                    End If
                Next s
            End If
        Next m
    Next n

    pbResizing = False
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   xAddResizeObject
'   PURPOSE:
'       - Adds a new object to the current collection
'       - Used during resizing of the form
'   PARAMETERS:
'       - Name (string)
'           - Name of the control to add to the resize collection
'       - Index (integer)
'           - Index of the control
'           - OR one of the following values:
'               FINDEX_ALL
'               FINDEX_NONE
'   RETURNS:
'       - None
Public Sub xAddResizeObject(rsName$, riIndex%)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xAddResizeObject"
    poError.xReset

    Dim n%, m%
    Dim bErrorChecked As Boolean
    Dim tTempControl As Control

'   See if the object exists on the bound form
    For n = 0 To poForm.Controls.Count - 1
        If (LCase$(poForm.Controls(n).Name) = LCase$(rsName)) Then
            bErrorChecked = False
            m = poForm.Controls(n).Index
            If (riIndex = FINDEX_ALL) Then
                Exit For
            ElseIf ((riIndex = FINDEX_NONE) And bErrorChecked) Then
                Set tTempControl = poForm.Controls(n)
                Exit For
            ElseIf (m = riIndex) Then
'               Found a specific control with the given name AND index
                Set tTempControl = poForm.Controls(n)
                Exit For
            End If
        End If
    Next n
    If (n > poForm.Controls.Count - 1) Then
        If (riIndex >= 0) Then
            poError.xSet sCurrentRoutine, "Unable to locate object '" & rsName & "(" & Format$(riIndex) & ")' on the form."
            Exit Sub
        Else
            poError.xSet sCurrentRoutine, "Unable to locate object '" & rsName & "' on the form."
            Exit Sub
        End If
    End If

    piNumObjects = piNumObjects + 1
    ReDim Preserve ptaResizeObject(1 To piNumObjects) As ResizeObjectType
    
    If (riIndex <> FINDEX_ALL) Then Set ptaResizeObject(piNumObjects).tObject = tTempControl
    ptaResizeObject(piNumObjects).sName = rsName
    ptaResizeObject(piNumObjects).iIndex = riIndex

    piCurResizeObject = piNumObjects

    Exit Sub
ERRORTRAP:
    Select Case Err.Number
    Case 343    'property Index doesn't exist
        bErrorChecked = True
    Case Else
        Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    Resume Next
End Sub

'   xAddStyle
'   PURPOSE:
'       - Adds a Style of the current ResizeObject
'   PARAMETERS:
'       - Style (integer)
'           - The style of resize to perform when a change is made length wise
'       - Align (optional control)
'           - Only used if alignment styles are selected
'   RETURNS:
'       - None
Public Sub xAddStyle(riStyle%, Optional rvntAlign)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xAddStyle"
    poError.xReset

    Dim n%

    If (piNumObjects <= 0) Then
        poError.xSet sCurrentRoutine, "Unable to add Style if no Resize Object exist."
        Exit Sub
    End If
    
    ptaResizeObject(piCurResizeObject).iNumStyles = ptaResizeObject(piCurResizeObject).iNumStyles + 1
    n = ptaResizeObject(piCurResizeObject).iNumStyles
    ReDim Preserve ptaResizeObject(piCurResizeObject).taStyle(1 To n) As StyleType
    
    Select Case riStyle
    Case FRESIZE_ALIGN_ABOVE, FRESIZE_ALIGN_BELOW, FRESIZE_ALIGN_LEFT, FRESIZE_ALIGN_RIGHT
        If IsMissing(rvntAlign) Then
            poError.xSet sCurrentRoutine, "Can't add an Align Style without being provided a control to align to."
            Exit Sub
        End If
        If (ptaResizeObject(piCurResizeObject).iIndex = FINDEX_ALL) Then
            poError.xSet sCurrentRoutine, "Can't add an Align Style to an array of resize objects."
            Exit Sub
        End If
        
        Set ptaResizeObject(piCurResizeObject).taStyle(n).tAlign = rvntAlign
        
        Select Case riStyle
        Case FRESIZE_ALIGN_ABOVE
            ptaResizeObject(piCurResizeObject).taStyle(n).iAlignValue = ptaResizeObject(piCurResizeObject).taStyle(n).tAlign.Top - (ptaResizeObject(piCurResizeObject).tObject.Top + ptaResizeObject(piCurResizeObject).tObject.Height)
        Case FRESIZE_ALIGN_BELOW
            ptaResizeObject(piCurResizeObject).taStyle(n).iAlignValue = ptaResizeObject(piCurResizeObject).tObject.Top - (ptaResizeObject(piCurResizeObject).taStyle(n).tAlign.Top + ptaResizeObject(piCurResizeObject).taStyle(n).tAlign.Height)
        Case FRESIZE_ALIGN_LEFT
            ptaResizeObject(piCurResizeObject).taStyle(n).iAlignValue = ptaResizeObject(piCurResizeObject).taStyle(n).tAlign.Left - (ptaResizeObject(piCurResizeObject).tObject.Left + ptaResizeObject(piCurResizeObject).tObject.Width)
        Case FRESIZE_ALIGN_RIGHT
            ptaResizeObject(piCurResizeObject).taStyle(n).iAlignValue = ptaResizeObject(piCurResizeObject).tObject.Left - (ptaResizeObject(piCurResizeObject).taStyle(n).tAlign.Left + ptaResizeObject(piCurResizeObject).taStyle(n).tAlign.Width)
        End Select
    End Select
    
    ptaResizeObject(piCurResizeObject).taStyle(n).iStyle = riStyle
    ptaResizeObject(piCurResizeObject).taStyle(n).fPercent = 1
    
    ptaResizeObject(piCurResizeObject).iCurStyle = n
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub



Private Sub Class_Initialize()
    pbInitialized = False
End Sub

'   Class_Terminate
'   PURPOSE:
'       - Stores the forms location and size to the INI file
'   PARAMETERS:
'       - None
'   RETURNS:
'       - None
Private Sub Class_Terminate()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".Class_Terminate"
    poError.xReset
    
    Dim n%
    Dim sBuffer$
    
    sBuffer = "(2) '|'"
    sBuffer = sBuffer & " Top | " & Format$(poForm.Top)
    sBuffer = sBuffer & " | Left | " & Format$(poForm.Left)
    sBuffer = sBuffer & " | Width | " & Format$(poForm.Width)
    sBuffer = sBuffer & " | Height | " & Format$(poForm.Height)
    
    poIni.xSet "Forms", poForm.Name, sBuffer
    
    Set poForm = Nothing
    Set poError = Nothing
    Set poIni = Nothing
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


