Attribute VB_Name = "Globals"
Option Explicit

Public Const OBJ_NAME = "Globals"

Public Const DATABASE_NAME = "\database\database.mdb"

Public Const CATEGORY_DEFAULT = "(2) '|' Canned | 2 | Produce | 7 | Meat | 5 | Dairy | 3 | Candy | 1 | Pharam. | 6 | Magazine | 4 | Other | 8"
Public Const CATEGORY_DEFAULT_VAL = 8
Public Const BULKUNITS_DEFAULT = "(2) '|' cans | 2 | grams | 7 | lb | 5 | litres | 3 | gross | 1 | bottles | 6 | copies | 4 | gross | 8"
Public Const ORDERING_DEFAULT = "(2) '|' Auto | 0 | Manual | 1 | Approval | 2"
Public Const ORDERING_AUTO = 0

Public Const STATUS_DEFAULT = "(2) '|' Pending | 0 | Filled | 1 | Incomplete | 2 | Received | 3"
Public Const STATUS_PENDING = 0
Public Const STATUS_FILLED = 1
Public Const STATUS_INCOMPLETE = 2
Public Const STATUS_RECEIVED = 3

'   Security permissions
Public Const ALLPERMISSIONS = "'|' Warehouse | All Stores | Security | Personnel | AutoOrder | ManualOrder | UpdateOrder | Exit | Inventory | Cashier | Printing"
Public Const PERMISSION_WAREHOUSE = 1
Public Const PERMISSION_STORES = 2
Public Const PERMISSION_SECURITY = 3
Public Const PERMISSION_CHANGEPERSONNEL = 4
Public Const PERMISSION_AUTOORDER = 5
Public Const PERMISSION_MANUALORDER = 6
Public Const PERMISSION_CHANGEORDERS = 7
Public Const PERMISSION_EXIT = 8
Public Const PERMISSION_INVENTORY = 9
Public Const PERMISSION_CASHIER = 10
Public Const PERMISSION_PRINTING = 11

Public goDBase As DBaseObj
Public goError As ErrorObj
Public goIni As IniObj

Public Type SystemTYPE
    iEmployeeID As Integer                  ' The current user's employee ID
    sUserName As String                     ' The current UserName
    iCategory As Integer                    ' The default category on new products
    sCategory As String                     ' The product category types
    sBulkUnits As String                    ' The bulk units of each category type
    sOrdering As String                     ' The inventory ordering types
    iStoreID As Integer                     ' The current StoreID (from Employee)
    sPermittedCategories As String          ' The permitted categories of user
    sPermissions As String                  ' The permissions of the current user
    iSecurity As Integer                    ' The security type of the user
    sAllPermissions As String               ' All the permissions
    sStatus As String                       ' All the order statuses
End Type
Public gtSystem As SystemTYPE



'   xSelection
'   PURPOSE:
'       - Handles the selection buttons that interact with the grid
'   PARAMETERS:
'       - Grid (GridObj)
'           - The Grid object to manipulate
'       - ButtonType (string)
'           - The button type (name of the button)
'   RETURNS:
'       - (boolean)
'           - TRUE : Unload the form
'           - FALSE: Don't unload the form
Public Function xSelection(roGrid As GridObj, roControl As Control) As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xSelection"
    Dim bErrorChecked As Boolean: bErrorChecked = False
    
    xSelection = False
    Select Case LCase$(roControl.Caption)
    Case "&exit"
        xSelection = True
    Case "&add"
'       Create a new empty line on the table and focus on it
        roGrid.xSort ""
        roGrid.xAddNew
    Case "&delete"
'       Delete the current line on the table
        roGrid.xDelete
    Case "&print"
'       Print the entire table
        MainMenu.CDialog.CancelError = True
        MainMenu.CDialog.ShowPrinter
        If bErrorChecked Then Exit Function
        
        MsgBox "Document will be printed at this point."
    Case "&sort by"
'       Sort the table
        SortBy.ShowForm roGrid
    Case "&find"
'       Locate a specific record in the table
        Find.ShowForm roGrid
        roControl.Caption = "&Reset"
    Case "&reset"
        roGrid.xMassFind ""
        roControl.Caption = "&Find"
    End Select
    
    Exit Function
ERRORTRAP:
    Select Case Err.Number
    Case 32755  'Cancel pressed
        bErrorChecked = True
    Case Else
        Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    Resume Next
End Function


'   IsPermissible
'   PURPOSE:
'       - Returns whether the current user has permission to enter a restricted
'         area of the application
'   PARAMETERS:
'       - Permission (integer)
'           - The permission required
'       - Permissions (optional string)
'           - The listing of permissions available
'           - If not given the current user permissions are used
'   RETURNS:
'       - (boolean)
'           - TRUE : The user has that permission
'           - FALSE: The user doesn't have that permission
Public Function IsPermissible(riPermission%, Optional rvntPermissions) As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".IsPermissible"

'   The permissions are stored in a 255 byte character string where each
'   character is a 4 bit hexadecimal value

    Dim iMod%, iDiv%, iReturn%

'   Get the proper character, then convert to numeric, and finally compare
    iMod = 2 ^ ((riPermission - 1) Mod 4)
    iDiv = Int(CSng(riPermission - 1) / 4) + 1
    
    If IsMissing(rvntPermissions) Then
        iReturn = HexToDec(Mid$(gtSystem.sPermissions, iDiv, 1))
    Else
        iReturn = HexToDec(Mid$(CStr(rvntPermissions), iDiv, 1))
    End If
    iReturn = (iReturn And iMod)
        
    IsPermissible = (iReturn > 0)

    Exit Function
ERRORTRAP:
    Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

'   IsPermittedCategory
'   PURPOSE:
'       - Returns whether the current user has permission to view certain categories
'   PARAMETERS:
'       - Permission (integer)
'           - The permission required
'       - Permissions (optional string)
'           - The listing of permissions available
'           - If not given the current user permissions are used
'   RETURNS:
'       - (boolean)
'           - TRUE : The user has that permission
'           - FALSE: The user doesn't have that permission
Public Function IsPermittedCategory(riPermission%, Optional rvntPermissions) As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".IsPermittedCategory"

'   The permissions are stored in a 255 byte character string where each
'   character is a 4 bit hexadecimal value

    Dim iMod%, iDiv%, iReturn%

'   Get the proper character, then convert to numeric, and finally compare
    iMod = 2 ^ ((riPermission - 1) Mod 4)
    iDiv = Int(CSng(riPermission - 1) / 4)
    
    If IsMissing(rvntPermissions) Then
        iReturn = HexToDec(Mid$(gtSystem.sPermittedCategories, iDiv + 1, 1))
    Else
        iReturn = HexToDec(Mid$(CStr(rvntPermissions), iDiv + 1, 1))
    End If
    iReturn = (iReturn And iMod)
        
    IsPermittedCategory = (iReturn > 0)

    Exit Function
ERRORTRAP:
    Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function


'   ReturnValidCategories
'   PURPOSE:
'       - Returns the valid categories for this user from the list of all categories
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - Returns an extractable listing of valid categories for this user
Public Function ReturnValidCategories$()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".ReturnValidCategories"

    Dim n%
    Dim tReturn As ExtractObj: Set tReturn = New ExtractObj
    Dim sBuffer$
    
    sBuffer = "(2) '|'"
    
    tReturn.gsValue = gtSystem.sCategory
    For n = 1 To tReturn.giNumValues
        If IsPermittedCategory(Val(tReturn.gsaList(2, n))) Then
            sBuffer = sBuffer & " " & tReturn.gsaList(1, n) & " | " & tReturn.gsaList(2, n) & " | "
        End If
    Next n
    ReturnValidCategories = LeftChop$(sBuffer, 3)

    Exit Function
ERRORTRAP:
    Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

'   AutoOrderIt
'   PURPOSE:
'       - Creates an order for all inventory that is below
'         its alert level
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (integer)
'           - The number of the order created
'           - -1 if no order was made
Public Function AutoOrderIt%()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".AutoOrderIt"

    goIni.xSet "General", "LastOrder", Date
    
    Dim tDBase As DBaseObj
    Dim piOrderNumber%
    Dim sBuffer$
    Dim lOnOrder&
    
    Set tDBase = New DBaseObj
    
    goDBase.gsCurTable = "Detail"
    piOrderNumber = Val(goDBase.xMax("OrderNumber")) + 1
    
    tDBase.xInitialize App.Path & DATABASE_NAME
    tDBase.xAddTable "Inventory", ""
    
    tDBase.xMassFind "(quantity + onorder) <= alertlevel AND ordering = " & Format$(ORDERING_AUTO)
    
    If tDBase.xMoveFirst Then
        Do
            lOnOrder = Val(tDBase.xGet("OnOrder"))
            lOnOrder = lOnOrder + Val(tDBase.xGet("RestockQty"))
            tDBase.xSet "OnOrder", lOnOrder
            
            sBuffer = "(2) '|'"
            sBuffer = sBuffer & " StoreID | " & Format$(gtSystem.iStoreID)
            sBuffer = sBuffer & " | OrderNumber | " & Format$(piOrderNumber)
            sBuffer = sBuffer & " | UPC | " & Format$(tDBase.xGet("UPC"))
            sBuffer = sBuffer & " | AmtOrdered | " & Format$(tDBase.xGet("RestockQty"))
            goDBase.xAddNew sBuffer
        Loop While (tDBase.xMoveNext)
    
        goDBase.gsCurTable = "Orders"
        
        sBuffer = "(2) '|'"
        sBuffer = sBuffer & " OrderNumber | " & Format$(piOrderNumber)
        sBuffer = sBuffer & " | StoreID | " & Format$(gtSystem.iStoreID)
        sBuffer = sBuffer & " | EmployeeID | -1"
        sBuffer = sBuffer & " | DateOrdered | " & Format$(Now)
        
        goDBase.xAddNew sBuffer
    
        AutoOrderIt = piOrderNumber
    Else
        AutoOrderIt = -1
    End If

    Exit Function
ERRORTRAP:
    Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

'   ReturnPossibleLevels
'   PURPOSE:
'       - Returns the security levels that can be selected by the current user
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - Extractable listing of all valid security levels for current user
Public Function ReturnPossibleLevels()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".ReturnPossibleLevels"
    
    Dim n%
    Dim sList$: sList = "(2) '|' "
    Dim tPReturn As ExtractObj: Set tPReturn = New ExtractObj
    Dim tCReturn As ExtractObj: Set tCReturn = New ExtractObj
    Dim sPermission$
    
    tPReturn.gsValue = gtSystem.sAllPermissions
    tCReturn.gsValue = gtSystem.sCategory
    
    goDBase.gsCurTable = "Security"
    
    If goDBase.xMoveFirst Then
'       Check each security level and see if the current level can make it
        Do
            sPermission = CStr(goDBase.xGet("Access"))
            For n = 1 To tPReturn.giNumValues
                If IsPermissible(Val(tPReturn.gsaList$(2, n)), sPermission) Then
'                   It's permitted by the selected security
                    If Not IsPermissible(Val(tPReturn.gsaList$(2, n))) Then
'                       It's NOT permitted by the current user's security
'                       This security is invalid, don't put it in
                        Exit For
                    End If
                End If
            Next n
            If (n <= tPReturn.giNumValues) Then GoTo NEXT_SECURITY

            sPermission = CStr(goDBase.xGet("Categories"))
            For n = 1 To tCReturn.giNumValues
                If IsPermittedCategory(Val(tCReturn.gsaList$(2, n)), sPermission) Then
'                   It's permitted by the selected security
                    If Not IsPermittedCategory(Val(tCReturn.gsaList$(2, n))) Then
'                       It's NOT permitted by the current user's security
'                       This security is invalid, don't put it in
                        Exit For
                    End If
                End If
            Next n
            If (n <= tCReturn.giNumValues) Then GoTo NEXT_SECURITY
            
'           Add this security level to the outgoing securities
            sList = sList & Format$(goDBase.xGet("Description")) & " | " & Format$(goDBase.xGet("Security")) & " | "

NEXT_SECURITY:
        Loop While goDBase.xMoveNext
    End If
    
    ReturnPossibleLevels = Left$(sList, Len(sList) - 3)
    
    Exit Function
ERRORTRAP:
    Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

'   UpdateDetail
'   PURPOSE:
'       - Updates a change in an order detail
'   PARAMETERS:
'       - UPC (string)
'           - The UPC value of the inventory item to update
'       - Change (long)
'           - The change in the quantity
'   RETURNS:
'       - None
Public Sub UpdateDetail(rsUPC$, rlChange&)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".UpdateDetail"
    
    Dim lQuantity&
    Dim sBuffer$
    Dim tDBase As DBaseObj

    Set tDBase = New DBaseObj
                    
    tDBase.xInitialize App.Path & DATABASE_NAME
    tDBase.xAddTable "Product", ""
    tDBase.xAddTable "Inventory", ""
    
    tDBase.gsCurTable = "Inventory"
    If tDBase.xFind("UPC = '" & rsUPC & "'") Then
'       Change the number of products in stock by given change
        lQuantity = Val(tDBase.xGet("Quantity"))
        lQuantity = lQuantity + rlChange
        tDBase.xSet "Quantity", lQuantity
        
'       Change the number of products "on order"
        lQuantity = Val(tDBase.xGet("OnOrder"))
        lQuantity = lQuantity - rlChange
                        
        If (lQuantity < 0) Then lQuantity = 0
                        
        tDBase.xSet "OnOrder", lQuantity
    Else
'       Create a new record for this product
        sBuffer = "(2) '|'"
        sBuffer = sBuffer & " UPC | " & rsUPC
        sBuffer = sBuffer & " | StoreID | " & Format$(gtSystem.iStoreID)
        sBuffer = sBuffer & " | Quantity | " & Format$(rlChange)
        
        tDBase.gsCurTable = "Product"
                        
        If tDBase.xFind("UPC = '" & rsUPC & "'") Then
            sBuffer = sBuffer & " | SalesPrice | " & Format$(tDBase.xGet("UnitCost"))
        End If
        
        tDBase.gsCurTable = "Inventory"
        
        tDBase.xAddNew sBuffer
        
        MsgBox "A new inventory record was created for UPC: '" & rsUPC & "'.  Make sure to change default data IMMEDIATELY."
'        Inventory.Show 1
    End If

    Exit Sub
ERRORTRAP:
    Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub
