VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GridObj"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'   BINDING RULES
'----------------------------------------------------------------------------------

'   Object Initialization
'       <GridObj object>.xInitialize <Form control>, <TrueGrid control>, <database name>, <data control>
'
'   poGrid (TrueGrid control)
'       EnterEdit
'           <GridObj object>.xEnterEdit
'       ExitEdit
'           <GridObj object>.xExitEdit
'       KeyPress
'           <GridObj object>.xSetKey KeyAscii
'       Validate
'           Cancel = Not <GridObj object>.xValidate(<TrueGrid control>.ColumnField(Col), Value)

'----------------------------------------------------------------------------------
'   CONSTANTS
'----------------------------------------------------------------------------------

Private Const OBJ_NAME = "GridObj"

'----------------------------------------------------------------------------------
'   OBJECTS
'----------------------------------------------------------------------------------

Private poDBase As New DBaseObj         ' The database object
Private poGrid As TrueGrid              ' The grid object
Private poForm As Form                  ' The form that is bound to this object
Private poError As New ErrorObj         ' The error handling routines
Private poIni As New IniObj             ' The error handling routines

'----------------------------------------------------------------------------------
'   STRUCTURES
'----------------------------------------------------------------------------------

Private Type BookMarkTYPE
    lBookMark As Long                   ' The Bookmark itself
    bIsEmpty As Boolean                 ' Whether this position is in use
End Type

'----------------------------------------------------------------------------------
'   VARIABLES
'----------------------------------------------------------------------------------

Private ptaBookMark() As BookMarkTYPE   ' A listing of current bookmarks
Private piNumBookMarks As Integer       ' The number of current bookmarks

Private psTableName$                    ' The name of the table bound to grid

Private pbaEditReenable() As Boolean    ' A listing of the controls to reenable
                                        ' after an edit
Private psNotWhileEdit$                 ' Controls to disable while in edit mode

Private piRowIndex%                     ' The current row selected
Private pbValidated As Boolean          ' Whether the current edit has been validated
Private pbIsGoodData As Boolean         ' Whether the validation was good (or bad)
Private psOldValue As String            ' The value before the current edit

Private piNumRules As Integer           ' Number of column validations required to edit
Private psaRule() As String             ' The columns required to be validated

Private pbInEditMode As Boolean         ' Currently in edit mode or not
Private pbEscPressed As Boolean         ' Whether ESC was pressed during edit mode

Private pbInitialized As Boolean            ' Whether xInitialize has been run

'   goVForm (Set)
'   PURPOSE:
'       - Sets the form to show the valid values list on
'       - Also makes certain that a list box exists in the form
'   PARAMETERS:
'       - Form (form control)
'           - The form to check
'   RETURNS:
'       - None
Public Property Set goVForm(roForm As Form)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".goVForm (Set)"
    poError.xReset

    Set poDBase.goVForm = roForm
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property

'   xFind
'   PURPOSE:
'       - Attempts to locate a record which matchs the given find criteria
'   PARAMETERS:
'       - Criteria (string)
'           - The criteria to find the record
'   RETURNS:
'       - (boolean)
'           - TRUE : A record was located
'           - FALSE: A record was not located
Public Function xFind(rsCriteria$) As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xFind"
    poError.xReset
                        
    Dim iIndex%
    
    xFind = poDBase.xFind(rsCriteria)
    
    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function



'   xReQuery
'   PURPOSE:
'       - Requeries the database to make a new recordset
'       - Uses SortCriteria, FindCriteria, RequiredFind
'   PARAMETERS:
'       - None
'   RETURNS:
'       - None
Public Sub xReQuery()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xReQuery"
    poError.xReset
    
    poDBase.xQuery
    xRefresh

    Exit Sub
ERRORTRAP:
    Select Case Err.Number
    Case 91     ' Recordset not set
'       Caused by the first call to this routine when the recordset is closed
'       even though it was never openned before hand
    Case Else
        Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    Resume Next
End Sub


'   gsFindFields (Get)
'   PURPOSE:
'       - Returns the Findable fields of the current recordset
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - The Findable fields in this recordset
Public Property Get gsFindFields$()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".gsFindFields (Get)"
    poError.xReset
    
    gsFindFields = poDBase.gsFindFields
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property



'   gsFindFields (Let)
'   PURPOSE:
'       - Sets the find fields of the current recordset
'   PARAMETERS:
'       - Fields (string)
'           - The find fields
'   RETURNS:
'       - None
Public Property Let gsFindFields(rsFields$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".gsFindFields (Let)"
    poError.xReset
        
    Dim n%
    Dim oReturn As ExtractObj: Set oReturn = New ExtractObj
    
'   Check the find fields data types
    oReturn.gsValue = rsFields
    For n = 1 To oReturn.giNumValues
        Select Case oReturn.gsaList(2, n)
        Case FIND_NUMERIC, FIND_TEXT, FIND_YES_NO, FIND_LIST
        Case Else
            poError.xSet sCurrentRoutine, "Undefined data type given: '" & oReturn.gsaList(2, n) & "'."
        End Select
    Next n
    
    poDBase.gsFindFields = rsFields
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property




'   xDisableField
'   PURPOSE:
'       - Makes the given field visible or invisible
'   PARAMETERS:
'       - Field (string)
'           - The field to change the visibility
'       - Visible (optional boolean)
'           - The visibility setting
'           - If not given the visibility is flipped
'   RETURNS:
'       - None
Public Sub xVisible(rsField$, Optional rvntVisible)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xVisible"
    poError.xReset

    Dim n%

    n = xGetColumn(rsField)
            
    If IsMissing(rvntVisible) Then
        poGrid.ColumnVisible(n) = Not poGrid.ColumnVisible(n)
    Else
        poGrid.ColumnVisible(n) = rvntVisible
    End If

    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub

'   xAddNew
'   PURPOSE:
'       - Adds a new line to the recordset and sets up the grid accordingly
'   PARAMETERS:
'       - Required (optional string)
'           - Contains a listing of the required values for certain fields
'   RETURNS:
'       - None
Public Sub xAddNew(Optional rvntRequired)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xAddNew"
    poError.xReset
    
    poDBase.gsCurTable = psTableName
    poDBase.xAddNew rvntRequired
    xRefresh
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   xiBookMark (Get)
'   PURPOSE:
'       - Bookmarks the current record on the grid
'       - Returns the index value
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (integer)
'           - The index value where the bookmark is stored
'           - '-1' is returned if no current record
Public Property Get xiBookMark%()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xiBookMark (Get)"
    poError.xReset
    Dim bErrorChecked As Boolean: bErrorChecked = False
    
    Dim n%
    
'   Locate an empty space in the bookmark listing
    For n = 1 To piNumBookMarks
        If ptaBookMark(n).bIsEmpty Then
            ptaBookMark(n).lBookMark = poGrid.RowIndex
            ptaBookMark(n).bIsEmpty = False
            Exit For
        End If
    Next n
    If (n > piNumBookMarks) Then
        piNumBookMarks = piNumBookMarks + 1
        ReDim Preserve ptaBookMark(1 To n) As BookMarkTYPE
    
        ptaBookMark(n).lBookMark = poGrid.RowIndex
        ptaBookMark(n).bIsEmpty = False
    End If
    xiBookMark = n
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property




'   xiBookMark (Let)
'   PURPOSE:
'       - sets the current record to the given bookmark
'       - Erases that bookmark
'   PARAMETERS:
'       - (integer)
'           - The index value where the bookmark is stored
'           - '-1' is ignored
'   RETURNS:
'       - None
Public Property Let xiBookMark(riIndex%)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xiBookMark (Let)"
    poError.xReset
    Dim bErrorChecked As Boolean: bErrorChecked = False
    
    If (riIndex = -1) Then Exit Property
    
    If ptaBookMark(riIndex).bIsEmpty Then
        poError.xSet sCurrentRoutine, "Attempt was made to retrieve an empty bookmark."
        Exit Property
    Else
'       Set the grid.rows property to the proper value by moving last
        If (poDBase.giNumRecords > 0) Then
            Do
                poGrid.RowIndex = ptaBookMark(riIndex).lBookMark
            Loop Until ((poGrid.RowIndex = ptaBookMark(riIndex).lBookMark) Or bErrorChecked)
        End If
        
        If (piNumBookMarks = riIndex) Then
'           Reduce the size of the Bookmark listing
            piNumBookMarks = piNumBookMarks - 1
            If (piNumBookMarks > 0) Then
                ReDim Preserve ptaBookMark(1 To piNumBookMarks) As BookMarkTYPE
            End If
        Else
'           Set to empty
            ptaBookMark(riIndex).bIsEmpty = True
        End If
    End If
    
    Exit Property
ERRORTRAP:
    Select Case Err.Number
    Case 380    'Invalid propert value
'       Caused by the RowIndex set (occasionally)
        bErrorChecked = True
    Case Else
        Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    Resume Next
End Property




'   gsValidList (Let)
'   PURPOSE:
'       - Sets the list of valid values for the current column
'   PARAMETERS:
'       - List (string)
'           - The possible values for this list
'   RETURNS:
'       - None
Public Property Let gsValidList(rsList$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsValidList (Let)"
    poError.xReset

    poDBase.gsValidList = rsList
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property




'   gsValidList (Get)
'   PURPOSE:
'       - Returns the list of valid values for the current column
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - The possible values for this list
Public Property Get gsValidList$()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsValidList (Get)"
    poError.xReset

    gsValidList = poDBase.gsValidList
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property





'   xSort
'   PURPOSE:
'       - Sorts the current recordset using the given SQL statement
'   PARAMETERS:
'       - Criteria (string)
'           - The sort criteria in standard SQL format
'   RETURNS:
'       - None
Public Sub xSort(rsCriteria$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xSort"
    poError.xReset
        
    poDBase.xSort rsCriteria
    xRefresh

    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub
'   gsSortCriteria (Get)
'   PURPOSE:
'       - Returns the sort criteria of the current recordset
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - The sort criteria in this recordset
Public Property Get gsSortCriteria$()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".gsSortCriteria (Get)"
    poError.xReset
    
    gsSortCriteria = poDBase.gsSortCriteria
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property






'   xMassFind
'   PURPOSE:
'       - Sets the recordset up to show only those records which match the
'         criteria
'   PARAMETERS:
'       - Criteria (string)
'           - The criteria to find the record
'   RETURNS:
'       - None
Public Sub xMassFind(rsCriteria$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xFind"
    poError.xReset
                
    poDBase.xMassFind rsCriteria
    xRefresh
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   xSetGridColList
'   PURPOSE:
'       - Sets the current column's list on the grid using the column list
'   PARAMETERS:
'       - List  (string)
'           - An extractable listing of the values this column can contain
'   RETURNS:
'       - None
Public Sub xSetGridColList(rsList$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xSetGridColList"
    poError.xReset

    Dim n%
    Dim oReturn As ExtractObj: Set oReturn = New ExtractObj
        
'   Load the category ValueList
    poGrid.VlistColumn = xGetColumn(poDBase.gsColumnName)
    poGrid.VlistCount = 0
        
    poDBase.gsColumnList = rsList
    poDBase.gsValidList = rsList
        
    oReturn.gsValue = rsList
    For n = 1 To oReturn.giNumValues
        poGrid.VlistText(n) = oReturn.gsaList(1, n)
        poGrid.VlistData(n) = Val(oReturn.gsaList(2, n))
    Next n

    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   xGet
'   PURPOSE:
'       - Returns the given field from the current record in the current table
'   PARAMETERS:
'       - rsField (string)
'           - The field to return
'   RETURNS:
'       - (variant)
'           - The value of the field
Public Function xGet(rsField$) As Variant
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xGet"
    poError.xReset
        
    xGet = poDBase.xGet(rsField)
    
    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function


'   xAddIndex
'   PURPOSE:
'       - Adds an index to the current recordset
'   PARAMETERS:
'       - IndexName (string)
'           - Name of the index (user-defined)
'       - Unique (boolean)
'           - Determines whether the index is unique or not
'   RETURNS:
'       - None
Public Sub xAddIndex(rsIndexName$, rbUnique As Boolean)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xAddIndex"
    poError.xReset

    poDBase.xAddIndex rsIndexName, rbUnique
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub



'   gsCurColumn (Let)
'   PURPOSE:
'       - Sets the current column of the current recordset
'   PARAMETERS:
'       - rsColumnName (string)
'           - The name of the column
'   RETURNS:
'       - None
Public Property Let gsCurColumn(rsColumnName$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsCurColumn (Let)"
    poError.xReset
    
    poDBase.gsCurColumn = rsColumnName
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property






'   gsIndexField (Let)
'   PURPOSE:
'       - Adds a field to the current index in the current recordset
'   PARAMETERS:
'       - FieldName (string)
'           - Name of the field
'   RETURNS:
'       - None
Public Property Let gsIndexField(rsFieldName$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".gsIndexField (Let)"
    poError.xReset

    poDBase.gsIndexField = rsFieldName
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property




'   xDelete
'   PURPOSE:
'       - Deletes the current record from the recordset
'   PARAMETERS:
'       - None
'   RETURNS:
'       - None
Public Sub xDelete()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xDelete"
    poError.xReset
    
    poDBase.gsCurTable = psTableName
    
    Dim iIndex%
    
    iIndex = xiBookMark
    poDBase.xDelete
    xRefresh
    xiBookMark = iIndex


    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub
        

'   xInitialize
'   PURPOSE:
'       - Sets up this class and the two sub objects
'   PARAMETERS:
'       - Form (Form)
'           - The form being bound to this object
'       - Table (TrueGrid)
'           - The grid to bind this object to
'       - DBaseName (string)
'           - The name of the database to open
'       - Data (data control)
'           - A link to a data control
'   RETURNS:
'       - None
Public Sub xInitialize(rtForm As Form, rtTable As TrueGrid, rsDBaseName$, rtData As Data)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = rtForm.Name & "." & OBJ_NAME & ".xInitialize"
    poError.xReset
    
    Dim oReturn As ExtractObj: Set oReturn = New ExtractObj
    Dim n%, m%
    Dim bErrorChecked As Boolean
    
    Set poGrid = rtTable
    Set poForm = rtForm
    
    poDBase.xInitialize rsDBaseName, rtData

'   Load any older values stored in the INI file
    n = poGrid.Index
    If bErrorChecked Then
        oReturn.gsValue = poIni.xGet("GridColumns", poForm.Name & "." & poGrid.Name, "")
    Else
        oReturn.gsValue = poIni.xGet("GridColumns", poForm.Name & "." & poGrid.Name & "(" & poGrid.Index & ")", "")
    End If
    For n = 1 To oReturn.giNumValues
        m = xGetColumn(oReturn.gsaList(1, n))
        If (m <> 0) Then
            poGrid.ColumnWidth(m) = Val(oReturn.gsaList(2, n))
            If (poGrid.ColumnWidth(m) <= 0) Then poGrid.ColumnWidth(m) = 1
        End If
    Next n

    Exit Sub
ERRORTRAP:
    Select Case Err.Number
    Case 343
        bErrorChecked = True
    Case Else
        Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    Resume Next
End Sub

'   xAddTable
'   PURPOSE:
'       - Opens the ONE table allowed in this object
'   PARAMETERS:
'       - TableName (string)
'           - The name of the table to open
'           - if SQL is given this is a user defined name given to the recordset
'       - Required (string)
'           - The required WHERE statement to make this recordset work
'           - "" is considered NO REQUIRED WHERE
'       - SQL (optional string)
'           - A SQL statement defining the recordset
'           - If missing, the table name is used instead
'   RETURNS:
'       - None
Public Sub xAddTable(rsTableName$, rsRequired$, Optional rvntSQL)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xAddTable"
    poError.xReset
           
    poDBase.xAddTable rsTableName, rsRequired, rvntSQL
    psTableName = rsTableName

    xRefresh
    xMoveFirst
        
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   xAddRule
'   PURPOSE:
'       - Adds a validation rule to the current column in the current recordset
'   PARAMETERS:
'       - Rule (integer)
'           - The rule to associate with the current column
'       - Value (optional long)
'           - The value to associate with the rule
'           - If not given, defaults to 0
'   RETURNS:
'       - None
Public Sub xAddRule(riRule%, Optional rvntValue)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xAddRule"
    poError.xReset

    poDBase.xAddRule riRule, rvntValue
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   gsValidationColumn (Let)
'   PURPOSE:
'       - Adds a validation column required before changes can occur on the grid
'   PARAMETERS:
'       - Column (string)
'           - The column to add to the validation check
'   RETURNS:
'       - None
Public Property Let gsValidationColumn(rsColumn$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".gsValidationColumn (Let)"
    poError.xReset
       
    piNumRules = piNumRules + 1
    ReDim Preserve psaRule$(1 To piNumRules)
    
'   See if column exists
    Call xGetColumn(rsColumn)
    
    psaRule(piNumRules) = rsColumn
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property



'   xAddColumn
'   PURPOSE:
'       - Adds a column to the current recordset
'   PARAMETERS:
'       - ColumnField (string)
'           - The field name of the column
'       - ColumnName (optional string)
'           - The header on the column
'           - This is used to distinguish between two columns which use
'             the same column field
'   RETURNS:
'       - None
Public Sub xAddColumn(rsColumnName$, Optional rvntColumnName)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xAddColumn"
    poError.xReset

'   Check to see if the Column given is valid
    If (xGetColumn(rsColumnName) = 0) Then
        poError.xSet sCurrentRoutine, "Unable to locate column '" & rsColumnName & "' on the grid."
        Exit Sub
    End If
    
    poDBase.xAddColumn rsColumnName, rvntColumnName
        
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   xMoveFirst
'   PURPOSE:
'       - Moves to the top of the grid
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (boolean)
'           - TRUE : A record was set as the current
'           - FALSE: No record could be set current
Public Function xMoveFirst() As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xMoveFirst"
    poError.xReset

    If (giNumRecords > 0) Then
        Do
            poGrid.RowIndex = 1
        Loop Until (poGrid.RowIndex = 1)
        xMoveFirst = True
    Else
        xMoveFirst = False
    End If
    
    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

'   xMoveLast
'   PURPOSE:
'       - Moves to the bottom of the grid
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (boolean)
'           - TRUE : A record was set as the current
'           - FALSE: No record could be set current
Public Function xMoveLast() As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xMoveLast"
    poError.xReset

    If (giNumRecords > 0) Then
        Do
            poGrid.RowIndex = giNumRecords
        Loop Until (poGrid.RowIndex = giNumRecords)
        xMoveLast = True
    Else
        xMoveLast = False
    End If
    
    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

'   xMoveNext
'   PURPOSE:
'       - Moves to the next record in the grid
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (boolean)
'           - TRUE : A record was set as the current
'           - FALSE: No record could be set current
Public Function xMoveNext() As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xMoveNext"
    poError.xReset

    If (poGrid.RowIndex = giNumRecords) Then
        xMoveNext = False
        Exit Function
    End If
    
    xMoveNext = True
    
    Dim lValue&: lValue = poGrid.RowIndex + 1
    Do
        poGrid.RowIndex = lValue
    Loop Until (poGrid.RowIndex = lValue)

    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function


'   giNumKeys (Let)
'   PURPOSE:
'       - Sets the maximum number of keys that can used in ONE sort
'   PARAMETERS:
'       - NumKeys (integer)
'           - The maximum number of keys that can used in ONE sort
'   RETURNS:
'       - None
Public Property Let giNumKeys(riNumKeys%)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".giNumKeys (Let)"
    poError.xReset
    
    poDBase.giNumKeys = riNumKeys
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property





'   giNumKeys (Get)
'   PURPOSE:
'       - Returns the maximum number of keys that can used in ONE sort
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (integer)
'           - The maximum number of keys that can used in ONE sort
Public Property Get giNumKeys%()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".giNumKeys (Get)"
    poError.xReset
    
    giNumKeys = poDBase.giNumKeys
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property






'   xMovePrev
'   PURPOSE:
'       - Moves to the previous record in the grid
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (boolean)
'           - TRUE : A record was set as the current
'           - FALSE: No record could be set current
Public Function xMovePrev() As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xMovePrev"
    poError.xReset

    If (poGrid.RowIndex = 1) Then
        xMovePrev = False
        Exit Function
    End If
    
    xMovePrev = True
    
    Dim lValue&: lValue = poGrid.RowIndex - 1
    Do
        poGrid.RowIndex = lValue
    Loop Until (poGrid.RowIndex = lValue)

    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function



'   gsSortFields (Get)
'   PURPOSE:
'       - Returns the sortable fields of the current recordset
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - The sortable fields in this recordset
Public Property Get gsSortFields$()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".gsSortFields (Get)"
    poError.xReset
        
    gsSortFields = poDBase.gsSortFields
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property


'   gsSortFields (Let)
'   PURPOSE:
'       - Sets the sortable fields of the current recordset
'   PARAMETERS:
'       - Fields (string)
'           - The sortable fields
'   RETURNS:
'       - None
Public Property Let gsSortFields(rsFields$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".gsSortFields (Let)"
    poError.xReset
    
    poDBase.gsSortFields = rsFields
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property


'   giNumRecords (Get)
'   PURPOSE:
'       - Returns the number of records in the current recordset
'   PARAMETERS:
'       - None
'   RETURNS:
'       - (string)
'           - The sortable fields in this recordset
Public Property Get giNumRecords%()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".giNumRecords (Get)"
    poError.xReset

    Dim iIndex%

    iIndex = xiBookMark
    giNumRecords = poDBase.giNumRecords
    xiBookMark = iIndex
    
    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property



'   Refresh
'   PURPOSE:
'       - Refreshs the data object (which refreshs the grid object)
'   PARAMETERS:
'       - None
'   RETURNS:
'       - None
Public Sub xRefresh()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xRefresh"
    poError.xReset

    Dim iIndex%
    
    iIndex = xiBookMark
    poDBase.xRefresh
    xiBookMark = iIndex
    DoEvents

    poGrid.Enabled = (giNumRecords <> 0)

    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub

'   xValidate
'   PURPOSE:
'       - Validates the given input on the basis of the rules set out for
'         the given column in the recordset
'       - Also validates given the grid's validation rules
'   PARAMETERS:
'       - Column (integer/string)
'           - If (Integer) then The index of the column on the table
'           - If (string) then The field in the table to validate the value for
'       - Value (string)
'           - The value to apply the rules to
'   RETURNS:
'       - (boolean)
'           - TRUE : The rules held for the value
'           - FALSE: A rule was broken by the value
Public Function xValidate(rvntColumn, rsValue$) As Boolean
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xValidate"
    poError.xReset
    
    Dim bReturn As Boolean
    Dim n%, m%
    Dim sColumn$, iColumn%
    
    If (LCase$(TypeName(rvntColumn)) = "integer") Then
        iColumn = CInt(rvntColumn)
        sColumn = poGrid.ColumnField(iColumn)
        If (poDBase.xGetColumn(sColumn) = 0) Then
            sColumn = poGrid.ColumnField(iColumn) & "(" & poGrid.ColumnName(iColumn) & ")"
            If (poDBase.xGetColumn(sColumn) = 0) Then
                poError.xSet sCurrentRoutine, "Unable to locate the field: '" & sColumn & "' in the database."
                bReturn = False
                GoTo END_SUB
            End If
        End If
    Else
        sColumn = CStr(rvntColumn)
    End If
    
'   Check to see if the row is editable by the grid validation rules
    For n = 1 To piNumRules
        m = xGetColumn(psaRule(n))
        
'       Don't perform this validation for the current column
        If (psaRule(n) <> sColumn) Then
            If Not poDBase.xValidate(psaRule(n), poGrid.ColumnText(m), poGrid.ColumnText(m)) Then
                bReturn = False
                GoTo END_SUB
            End If
        End If
    Next n
    
'   See if the data given is valid for the current column
    bReturn = poDBase.xValidate(sColumn, rsValue, psOldValue)
        
END_SUB:
    pbValidated = True
    
    pbIsGoodData = bReturn
    xValidate = bReturn
        
    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

'   xEnterEdit
'   PURPOSE:
'       - Disables all the buttons on the form when edit mode is entered
'   PARAMETERS:
'       - None
'   RETURNS:
'       - None
Public Sub xEnterEdit()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xEnterEdit"
    poError.xReset
    
    Dim n%, m%, I%
    Dim oReturn As ExtractObj: Set oReturn = New ExtractObj
           
    If pbInEditMode Then Exit Sub
    pbInEditMode = True
           
    pbEscPressed = False
           
'   If the current column is not editable don't do anything
    If ((poGrid.ColumnStyle(poGrid.ColumnIndex) And 8192) = 8192) Then Exit Sub
           
    oReturn.gsValue = psNotWhileEdit
    
    I = 0
    For n = 1 To oReturn.giNumValues
        For m = 0 To poForm.Controls.Count - 1
            If (poForm.Controls(m).Name = oReturn.gsaList(1, n)) Then
                I = I + 1
                ReDim Preserve pbaEditReenable(1 To I) As Boolean
                pbaEditReenable(I) = poForm.Controls(m).Enabled
                poForm.Controls(m).Enabled = False
            End If
        Next m
    Next n

'   Store the current row
    piRowIndex = poGrid.RowIndex
    psOldValue = poGrid.ColumnText(poGrid.ColumnIndex)
    pbValidated = False

    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub
'   xExitEdit
'   PURPOSE:
'       - Enables all the buttons on the form when edit mode is left
'       - Also checks to see if the input is valid
'   PARAMETERS:
'       - None
'   RETURNS:
'       - None
Public Sub xExitEdit()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xExitEdit"
    poError.xReset

    Dim n%, m%, I%
    Dim oReturn As ExtractObj: Set oReturn = New ExtractObj
       
    If Not pbValidated Or pbIsGoodData Or pbEscPressed Then
        oReturn.gsValue = psNotWhileEdit
        I = 0
        For n = 1 To oReturn.giNumValues
            For m = 0 To poForm.Controls.Count - 1
                If (poForm.Controls(m).Name = oReturn.gsaList(1, n)) Then
                    I = I + 1
                    poForm.Controls(m).Enabled = pbaEditReenable(I)
                End If
            Next m
        Next n
        
'       Save the new data
        poDBase.xUpdate
        
        pbInEditMode = False
    Else
        poGrid.RowIndex = piRowIndex
        poGrid.EditActive = True
        Exit Sub
    End If
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub


'   gsNotWhileEdit
'   PURPOSE:
'       - Adds a control name to a listing of controls to diable during edits
'   PARAMETERS:
'       - ControlName (string)
'           - The name of the control to add
'   RETURNS:
'       - None
Public Property Let gsNotWhileEdit(rsControlName$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".gsNotWhileEdit"
    poError.xReset

    Dim n%
    Dim oReturn As ExtractObj: Set oReturn = New ExtractObj

'   See if the control actually exists on the form
    For n = 0 To poForm.Controls.Count - 1
        If (poForm.Controls(n).Name = rsControlName) Then Exit For
    Next n
    If (n > poForm.Controls.Count - 1) Then
        poError.xSet sCurrentRoutine, "The control: '" & rsControlName & "' does not exist on the form."
        Exit Property
    End If
    
    If (psNotWhileEdit <> "") Then
'       See if the ControlName already exists in the listing
        oReturn.gsValue = psNotWhileEdit
        For n = 1 To oReturn.giNumValues
            If (rsControlName = oReturn.gsaList(1, n)) Then Exit Property
        Next n
        psNotWhileEdit = psNotWhileEdit & " | " & rsControlName
    Else
        psNotWhileEdit = "'|' " & rsControlName
    End If

    Exit Property
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Property


Private Sub Class_Initialize()
    pbInitialized = False
End Sub

'   Class_Terminate
'   PURPOSE:
'       - Invalidate any objects in this class (frees up the memory)
'       - Stores the grid's current column widths to the INI file
'   PARAMETERS:
'       - None
'   RETURNS:
'       - None
Private Sub Class_Terminate()
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".Class_Terminate"
    poError.xReset
    
    Dim n%, m%
    Dim sBuffer$
    Dim oReturn As ExtractObj: Set oReturn = New ExtractObj
    Dim bErrorChecked As Boolean
    Dim sColumnName$
    
    sBuffer = "(2) '|' "
    
    oReturn.gsValue = poIni.xGet("GridColumns", poForm.Name, "")
    For n = 1 To poGrid.Columns
        sColumnName = LCase$(poGrid.ColumnField(n) & "(" & poGrid.ColumnName(n) & ")")
        If (poDBase.xGetColumn(sColumnName) = 0) Then sColumnName = LCase$(poGrid.ColumnField(n))
        
        If (poGrid.ColumnWidth(n) <= 0) Then
'           Attempt to use the old value stored in the INI file
            For m = 1 To oReturn.giNumValues
                If (LCase$(oReturn.gsaList(1, m)) = sColumnName) Then
                    sBuffer = sBuffer & sColumnName & " | " & oReturn.gsaList(2, m) & " | "
                    Exit For
                End If
            Next m
            If (m > oReturn.giNumValues) Then
                sBuffer = sBuffer & sColumnName & " | " & Format$(poGrid.ColumnWidth(n)) & " | "
            End If
        Else
            sBuffer = sBuffer & sColumnName & " | " & Format$(poGrid.ColumnWidth(n)) & " | "
        End If
    Next n
    
    poIni.xSet "GridColumns", poForm.Name & "." & poGrid.Name & "(" & poGrid.Index & ")", Left$(sBuffer, Len(sBuffer) - 3)
    If bErrorChecked Then
        poIni.xSet "GridColumns", poForm.Name & "." & poGrid.Name, Left$(sBuffer, Len(sBuffer) - 3)
    End If
    
    Set poDBase = Nothing
    Set poGrid = Nothing
    Set poForm = Nothing
    Set poError = Nothing
    Set poIni = Nothing
    
    Exit Sub
ERRORTRAP:
    Select Case Err.Number
    Case 343
        bErrorChecked = True
    Case Else
        Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    
    Resume Next
End Sub


'   xSet
'   PURPOSE:
'       - Sets the given Field to the given value in the current table
'   PARAMETERS:
'       - rsField (string)
'           - The field to return
'       - (variant)
'           - The value of the field
'   RETURNS:
'       - None
Public Sub xSet(rsField$, rvntValue)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".xSet"
    poError.xReset
        
    poDBase.xSet rsField, rvntValue
    
    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub




'   xGetColumn
'   PURPOSE:
'       - Returns the grid number of the column with the given ColumnField
'   PARAMETERS:
'       - Column (string)
'           - The name of the column to find
'   RETURNS:
'       - (integer)
'           - The grid number of the column
Public Function xGetColumn%(rsColumn$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xGetColumn"
    poError.xReset

    Dim m%, n%
    Dim sColumn$

    If InStr(rsColumn, "(") Then
        n = InStr(rsColumn, "(") + 1
        m = InStr(rsColumn, ")") - 1
        sColumn = LCase$(Mid$(rsColumn, n, m - n + 1))
    End If

    For n = 1 To poGrid.Columns
        If InStr(rsColumn, "(") Then
            If (LCase$(poGrid.ColumnName(n)) = sColumn) Then Exit For
        Else
            If (LCase$(poGrid.ColumnField(n)) = LCase$(rsColumn)) Then Exit For
        End If
    Next n
    If (n > poGrid.Columns) Then n = 0

    xGetColumn = n

    Exit Function
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

'   xSetKey
'   PURPOSE:
'       - Picks up keys that are usefull, such as ESC and sets proper variables
'   PARAMETERS:
'       - KeyAscii (integer)
'           - The Ascii value of the key pressed
Public Sub xSetKey(riKeyAscii%)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = poForm.Name & "." & OBJ_NAME & ".xSetKey"
    poError.xReset

    Select Case riKeyAscii
    Case 27 'ESC
        pbEscPressed = True
    End Select

    Exit Sub
ERRORTRAP:
    Call poError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Sub
