Attribute VB_Name = "HexedBas"
Option Explicit

Private Const OBJ_NAME = "HexedBas"

Public Const HS_TO_ASC = "ASC"
Public Const HS_TO_HEX = "HEX"
'   HexAdd
'   PURPOSE:
'       - Adds two hexadecimal numbers together and returns a HEX value
'   PARAMETERS:
'       - Text1 (string)
'           - A Hexadecimal value
'       - Text2 (string)
'           - A Hexadecimal value
'   RETURNS:
'       - (string)
'           - The resulting Hexadecimal value
Public Function HexAdd$(rsText1$, rsText2$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".HexAdd"

    Dim lNum1&, lNum2&, n%
    Dim ch$

'   Convert HEX strings to decimal numbers, add, and reconvert
    lNum1 = HexToDec(rsText1$)
    lNum2 = HexToDec(rsText2$)

    If ((lNum1 = -1) Or (lNum2 = -1)) Then
'       HexToDec errored out
        HexAdd = -1
    Else
        HexAdd = Hex(lNum1 + lNum2)
    End If
    Exit Function
ERRORTRAP:
    Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

'   HexSub
'   PURPOSE:
'       - Subtract two hexadecimal numbers together and returns a HEX value
'   PARAMETERS:
'       - Text1 (string)
'           - A Hexadecimal value
'       - Text2 (string)
'           - A Hexadecimal value
'   RETURNS:
'       - (string)
'           - The resulting Hexadecimal value
Public Function HexSub$(rsText1$, rsText2$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".HexSub"

    Dim lNum1&, lNum2&, n%
    Dim ch$

'   Convert HEX strings to decimal numbers, subtract, and reconvert
    lNum1 = HexToDec(rsText1$)
    lNum2 = HexToDec(rsText2$)
    
    If ((lNum1 = -1) Or (lNum2 = -1)) Then
'       HexToDec errored out
        HexSub = "-1"
    ElseIf (lNum1 < lNum2) Then
'       The return value must not be a negative value
        HexSub = "-1"
    Else
        HexSub = Hex(lNum1 - lNum2)
    End If
    Exit Function
ERRORTRAP:
    Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function



Public Function HexToDec&(rsText$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = "HexToDec"

    Dim lNum&, n%, ch$, m%
    
    If (rsText = "-1") Then
'       This is used to pick up errors when HexSubtract errors out
        HexToDec = -1
        Exit Function
    End If
    
    lNum = 0
    For n = 1 To Len(rsText$)
        m = Len(rsText$) - n
        ch = UCase$(Mid$(rsText, n, 1))
        Select Case ch
        Case "0" To "9"
            lNum = lNum + (16 ^ m) * Val(ch)
        Case "A" To "F"
            lNum = lNum + (16 ^ m) * (Asc(ch) - Asc("A") + 10)
        End Select
    Next n
    
    HexToDec = lNum
    Exit Function
ERRORTRAP:
    Select Case Err.Number
    Case 6  'Overflow
'       Usually this occurs if the HEX number given was negative (ex. FFFFFFF8)
        HexToDec = -1
    Case Else
        Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    End Select
    Resume Next
End Function

'   HexCheckSum
'   PURPOSE:
'       - Returns the checksum value for a given hexadecimal value
'   PARAMETERS:
'       - Text (string)
'           - the Hexadecimal value
'   RETURNS:
'       - (string)
'           - The resulting Hexadecimal checksum value
'           - This will be one single character
Public Function HexCheckSum$(rsText$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".HexCheckSum"
    
'   Returns the Checksum in HEX for the given text

    Dim lCheckSum&, n%, ch$
        
'   Determine the checksum
    lCheckSum& = 0
    For n = 1 To Len(rsText$)
        ch = UCase$(Mid$(rsText$, n, 1))
        Select Case ch
        Case "0" To "9"
            lCheckSum& = lCheckSum& + Val(ch)
        Case "A" To "F"
            lCheckSum& = lCheckSum& + (Asc(ch) - Asc("A") + 10)
        End Select
    Next n
        
'   Return values are between 0 and 15
    lCheckSum& = lCheckSum& Mod 16
        
    HexCheckSum = Hex(lCheckSum&)
    Exit Function
ERRORTRAP:
    Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function





'   HexString
'   PURPOSE:
'       - Changes ascii characters to two character HEX, or vice versa
'       - This makes it possible to ReturnUnreadableText to work
'         on normal strings
'   PARAMETERS:
'       - Text (string)
'           - The text to convert
'       - HEXorASC (string)
'           - HEX: Change to hexadecimal
'           - ASC: Change to ascii
'   RETURNS:
'       - (string)
'           - The converted text
Public Function HexString$(rsText$, rsHEXorASC$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = OBJ_NAME & ".HexString"

    Dim n&, sText$
    
    Select Case UCase$(Trim$(rsHEXorASC$))
    Case "HEX", "HEXADECIMAL", "TOHEX", "FROMASCII", "TOH"
'       Change the given string to HEX
        sText = ""
        For n = 1 To Len(rsText$) Step 1
'           If the HEX number is not 2 characters, add a zero
            If (Len(Hex(Asc(Mid$(rsText$, n, 1)))) = 1) Then sText$ = sText$ & "0"
            sText$ = sText$ & Hex(Asc(Mid$(rsText$, n, 1)))
        Next n
    Case "ASC", "ASCII", "TOASCII", "FROMHEX", "TOA", "TOASC"
'       Change the given string to ASCII
'       Assume that all ascii characters are represented by two characters in rsText
        For n = 1 To Len(rsText$) Step 2
            sText$ = sText$ & Chr$(HexToDec(Mid$(rsText$, n, 2)))
        Next n
    End Select
    
    HexString = sText
    Exit Function
ERRORTRAP:
    Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function
Public Function ReturnReadableText(ByVal vsText$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = "ReturnReadableText"

'   Effectively removes all the flim-flam that ReturnUnreadableSN did
'   The values are HEXADECIMAL (0-9,A-F) or in ASCII (48-57,65-70)

    Dim n%, m%, I%
    Dim iNum1%, iNum2%
    Dim sText$
    
'   First determine the number (see ReturnUnReadableSN)
'   Minus it from the third, and then move onto the next set
    sText = Left$(vsText, 2)
    For n = 3 To Len(vsText$)
        iNum1 = Asc(Mid$(sText$, n - 2, 1)) - 128
        iNum2 = Asc(Mid$(sText$, n - 1, 1)) - 128
        sText = sText & Chr$(Asc(Mid$(vsText$, n, 1)) - Int((iNum1 ^ 2 + iNum2 ^ 2) ^ 0.5))
    Next n
    vsText = sText
    
'   Push all text down back to readable level
    sText$ = ""
    For n = 1 To Len(vsText$)
        I = Asc(Mid$(vsText$, n, 1))
        Select Case I
            Case 128 To 137 '0 to 9
                sText$ = sText$ & Chr$((I - 128) + Asc("0"))
            Case 138 To 143 'A to F
                sText$ = sText$ & Chr$((I - 138) + Asc("A"))
            Case 144  ' "-"
                sText$ = sText$ & "-"
        End Select
    Next n
    
    ReturnReadableText = sText$
    Exit Function
ERRORTRAP:
    Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function




Public Function ReturnUnreadableText(ByVal vsText$)
    On Error GoTo ERRORTRAP
    Dim sCurrentRoutine$: sCurrentRoutine$ = "ReturnUnreadableText"

'   Pushes the Serial Number into the unreadable character section of the
'       ASCII character set to make it more difficult to plug in
'   Make sure that the text string given is uppercase

    vsText = UCase$(vsText$)

'   The values are HEXADECIMAL (0-9,A-F) or in ASCII (48-57,65-70)

    Dim n%, m%
    Dim iNum1%, iNum2%
    Dim sText$, ch$

'   Push all text up above 127 DECIMAL (0-9,A-F) -> ASCII (128-137,138-143)
    sText$ = ""
    For n = 1 To Len(vsText$)
        ch = Mid$(vsText$, n, 1)
        Select Case ch
        Case "0" To "9"
            sText$ = sText$ & Chr$((Asc(ch) - Asc("0")) + 128)
        Case "A" To "F"
            sText$ = sText$ & Chr$((Asc(ch) - Asc("A")) + 138)
        Case "-"
            sText$ = sText$ & Chr$(144)
        End Select
    Next n
    vsText$ = sText$
    
'   Now do a little playing around to screw up anyone who would have the gaul
'       to attempt to read this new number
'   RELATIVE numbers range from 0 to 16
'   Take the first,second numbers
'       Get the RELATIVE numbers by minusing 128 from each
'           Square each number, add them, and then square root them
'           Then add the result to the third number
'       Do this for each group of three numbers (using the original numbers)
    sText = Left$(vsText$, 2)
    For n = 3 To Len(vsText$)
        iNum1 = Asc(Mid$(vsText$, n - 2, 1)) - 128
        iNum2 = Asc(Mid$(vsText$, n - 1, 1)) - 128
        sText = sText & Chr$(Asc(Mid$(vsText$, n, 1)) + Int((iNum1 ^ 2 + iNum2 ^ 2) ^ 0.5))
    Next n

    ReturnUnreadableText = sText$
    Exit Function
ERRORTRAP:
    Call goError.xLog(Err.Number, sCurrentRoutine, Err.Description)
    Resume Next
End Function

